@@ -44,10 +44,10 @@ import org.apache.tools.ant.BuildException;
  * <ul>
  * <li>
  * Get a session attribute hello from session with ref <em>${sessionid.0}</em> form 
- * app <em>Catalina:type=Manager,path=/ClusterTest,host=localhost</em> 
+ * app <em>Catalina:type=Manager,context=/ClusterTest,host=localhost</em> 
  * <pre>
  *   &lt;jmx:invoke
- *           name="Catalina:type=Manager,path=/ClusterTest,host=localhost" 
+ *           name="Catalina:type=Manager,context=/ClusterTest,host=localhost" 
  *           operation="getSessionAttribute"
  *           resultproperty="hello"&gt;
  *         &lt;arg value="${sessionid.0}"/&gt;
@@ -60,8 +60,8 @@ import org.apache.tools.ant.BuildException;
  * <code>
  *   &lt;jmx:invoke
  *           name="Catalina:type=MBeanFactory" 
- *           operation="createAcccesLoggerValve"
- *           resultproperty="acccesLoggerObjectName"
+ *           operation="createAccessLoggerValve"
+ *           resultproperty="accessLoggerObjectName"
  *       &gt;
  *         &lt;arg value="Catalina:type=Host,host=localhost"/&gt;
  *   &lt;/jmx:invoke&gt;
@@ -87,7 +87,7 @@ import org.apache.tools.ant.BuildException;
  * These tasks require Ant 1.6 or later interface.
  *
  * @author Peter Rossbach
- * @version $Id: JMXAccessorInvokeTask.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: JMXAccessorInvokeTask.java 1060524 2011-01-18 18:38:02Z kkolinko $
  * @since 5.5.10
  */
 
@@ -97,7 +97,7 @@ public class JMXAccessorInvokeTask extends JMXAccessorTask {
     // ----------------------------------------------------- Instance Variables
 
     private String operation ;
-    private List args=new ArrayList();
+    private List<Arg> args=new ArrayList<Arg>();
 
     // ----------------------------------------------------- Instance Info
 
@@ -111,6 +111,7 @@ public class JMXAccessorInvokeTask extends JMXAccessorTask {
      * corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -139,13 +140,13 @@ public class JMXAccessorInvokeTask extends JMXAccessorTask {
     /**
      * @return Returns the args.
      */
-    public List getArgs() {
+    public List<Arg> getArgs() {
         return args;
     }
     /**
      * @param args The args to set.
      */
-    public void setArgs(List args) {
+    public void setArgs(List<Arg> args) {
         this.args = args;
     }
     
@@ -159,6 +160,7 @@ public class JMXAccessorInvokeTask extends JMXAccessorTask {
      * @exception BuildException
      *                if an error occurs
      */
+    @Override
     public String jmxExecute(MBeanServerConnection jmxServerConnection)
         throws Exception {
 
@@ -185,7 +187,7 @@ public class JMXAccessorInvokeTask extends JMXAccessorTask {
             Object argsA[]=new Object[ args.size()];
             String sigA[]=new String[args.size()];
             for( int i=0; i<args.size(); i++ ) {
-                Arg arg=(Arg)args.get(i);
+                Arg arg=args.get(i);
                 if( arg.type==null) {
                     arg.type="java.lang.String";
                     sigA[i]=arg.getType();
