@@ -27,24 +27,20 @@ import java.util.Map;
 import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
 
-import org.apache.catalina.Lifecycle;
-import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Realm;
 import org.apache.catalina.Session;
 import org.apache.catalina.SessionEvent;
 import org.apache.catalina.SessionListener;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.valves.ValveBase;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
  * A <strong>Valve</strong> that supports a "single sign on" user experience,
  * where the security identity of a user who successfully authenticates to one
- * web application is propogated to other web applications in the same
+ * web application is propagated to other web applications in the same
  * security domain.  For successful use, the following requirements must
  * be met:
  * <ul>
@@ -59,13 +55,15 @@ import org.apache.catalina.valves.ValveBase;
  * </ul>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 536380 $ $Date: 2007-05-09 01:49:56 +0200 (Mi, 09. Mai 2007) $
+ * @version $Id: SingleSignOn.java 1188401 2011-10-24 21:50:26Z markt $
  */
 
-public class SingleSignOn
-    extends ValveBase
-    implements Lifecycle, SessionListener {
+public class SingleSignOn extends ValveBase implements SessionListener {
 
+    //------------------------------------------------------ Constructor
+    public SingleSignOn() {
+        super(true);
+    }
 
     // ----------------------------------------------------- Instance Variables
 
@@ -81,15 +79,10 @@ public class SingleSignOn
     /**
      * Descriptive information about this Valve implementation.
      */
-    protected static String info =
+    protected static final String info =
         "org.apache.catalina.authenticator.SingleSignOn";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
     /**
      * Indicates whether this valve should require a downstream Authenticator to
      * reauthenticate each request, or if it itself can bind a UserPrincipal
@@ -107,15 +100,11 @@ public class SingleSignOn
     /**
      * The string manager for this package.
      */
-    protected final static StringManager sm =
+    @Deprecated
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Component started flag.
-     */
-    protected boolean started = false;
-
     /**
      * Optional SSO cookie domain.
      */
@@ -139,9 +128,10 @@ public class SingleSignOn
      */
     public void setCookieDomain(String cookieDomain) {
         if (cookieDomain != null && cookieDomain.trim().length() == 0) {
-            cookieDomain = null;
+            this.cookieDomain = null;
+        } else {
+            this.cookieDomain = cookieDomain;
         }
-        this.cookieDomain = cookieDomain;
     }
 
     /**
@@ -213,84 +203,6 @@ public class SingleSignOn
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
-    /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that prevents this component from being used
-     */
-    public void start() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (started)
-            throw new LifecycleException
-                (sm.getString("authenticator.alreadyStarted"));
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
-
-    }
-
-
-    /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
-     */
-    public void stop() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException
-                (sm.getString("authenticator.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
-
-    }
-
-
     // ------------------------------------------------ SessionListener Methods
 
 
@@ -299,6 +211,7 @@ public class SingleSignOn
      *
      * @param event SessionEvent that has occurred
      */
+    @Override
     public void sessionEvent(SessionEvent event) {
 
         // We only care about session destroyed events
@@ -313,7 +226,7 @@ public class SingleSignOn
 
         String ssoId = null;
         synchronized (reverse) {
-            ssoId = (String) reverse.get(session);
+            ssoId = reverse.get(session);
         }
         if (ssoId == null)
             return;
@@ -323,7 +236,7 @@ public class SingleSignOn
         // SSO.  If the session was logged out, we'll log out
         // of all session associated with the SSO.
         if (((session.getMaxInactiveInterval() > 0)
-            && (System.currentTimeMillis() - session.getLastAccessedTimeInternal() >=
+            && (System.currentTimeMillis() - session.getThisAccessedTimeInternal() >=
                 session.getMaxInactiveInterval() * 1000)) 
             || (Session.SESSION_PASSIVATED_EVENT.equals(event.getType()))) {
             removeSession(ssoId, session);
@@ -343,6 +256,7 @@ public class SingleSignOn
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -359,6 +273,7 @@ public class SingleSignOn
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet error occurs
      */
+    @Override
     public void invoke(Request request, Response response)
         throws IOException, ServletException {
 
@@ -423,25 +338,6 @@ public class SingleSignOn
     }
 
 
-    // --------------------------------------------------------- Public Methods
-
-
-    /**
-     * Return a String rendering of this object.
-     */
-    public String toString() {
-
-        StringBuffer sb = new StringBuffer("SingleSignOn[");
-        if (container == null )
-            sb.append("Container is null");
-        else
-            sb.append(container.getName());
-        sb.append("]");
-        return (sb.toString());
-
-    }
-
-
     // ------------------------------------------------------ Protected Methods
 
 
@@ -480,16 +376,16 @@ public class SingleSignOn
         }
 
         SingleSignOnEntry sso = lookup(ssoId);
-        if ( sso == null )
+        if (sso == null)
             return;
 
-        sso.removeSession( session );
+        sso.removeSession(session);
 
         // see if we are the last session, if so blow away ssoId
         Session sessions[] = sso.findSessions();
-        if ( sessions == null || sessions.length == 0 ) {
+        if (sessions == null || sessions.length == 0) {
             synchronized (cache) {
-                sso = (SingleSignOnEntry) cache.remove(ssoId);
+                cache.remove(ssoId);
             }
         }
 
@@ -510,7 +406,7 @@ public class SingleSignOn
         // Look up and remove the corresponding SingleSignOnEntry
         SingleSignOnEntry sso = null;
         synchronized (cache) {
-            sso = (SingleSignOnEntry) cache.remove(ssoId);
+            sso = cache.remove(ssoId);
         }
 
         if (sso == null)
@@ -661,7 +557,7 @@ public class SingleSignOn
     protected SingleSignOnEntry lookup(String ssoId) {
 
         synchronized (cache) {
-            return ((SingleSignOnEntry) cache.get(ssoId));
+            return cache.get(ssoId);
         }
 
     }
