@@ -74,6 +74,7 @@ import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.util.ParameterMap;
 import org.apache.catalina.util.StringParser;
 import org.apache.coyote.ActionCode;
+import org.apache.coyote.http11.upgrade.UpgradeInbound;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
@@ -99,7 +100,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Id: Request.java 1241170 2012-02-06 20:45:52Z markt $
+ * @version $Id: Request.java 1335733 2012-05-08 20:07:23Z markt $
  */
 
 public class Request
@@ -1007,9 +1008,9 @@ public class Request
      * The underlying connector may also expose request attributes. These all
      * have names starting with "org.apache.tomcat" and include:
      * <ul>
-     * <li>org.apache.tomcat.sendfile.support</li>
-     * <li>org.apache.tomcat.comet.support</li>
-     * <li>org.apache.tomcat.comet.timeout.support</li>
+     * <li>{@link Globals#SENDFILE_SUPPORTED_ATTR}</li>
+     * <li>{@link Globals#COMET_SUPPORTED_ATTR}</li>
+     * <li>{@link Globals#COMET_TIMEOUT_SUPPORTED_ATTR}</li>
      * </ul>
      * Connector implementations may return some, all or none of these
      * attributes and may also support additional attributes.
@@ -1514,7 +1515,7 @@ public class Request
 
         // Do the security check before any updates are made
         if (Globals.IS_SECURITY_ENABLED &&
-                name.equals("org.apache.tomcat.sendfile.filename")) {
+                name.equals(Globals.SENDFILE_FILENAME_ATTR)) {
             // Use the canonical file name to avoid any possible symlink and
             // relative path issues
             String canonicalPath;
@@ -2664,6 +2665,7 @@ public class Request
         }
 
         Parameters parameters = coyoteRequest.getParameters();
+        parameters.setLimit(getConnector().getMaxParameterCount());
 
         boolean success = false;
         try {
@@ -2802,6 +2804,21 @@ public class Request
         return null;
     }
 
+
+    // --------------------------------------------------------- Upgrade Methods
+
+    public void doUpgrade(UpgradeInbound inbound)
+            throws IOException {
+
+        coyoteRequest.action(ActionCode.UPGRADE, inbound);
+
+        // Output required by RFC2616. Protocol specific headers should have
+        // already been set.
+        response.setStatus(HttpServletResponse.SC_SWITCHING_PROTOCOLS);
+        response.flushBuffer();
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
