@@ -22,11 +22,12 @@ package org.apache.catalina.ha;
  * different deployment implementations
  *
  * @author Filip Hanik
- * @version $Id: ClusterDeployer.java 939514 2010-04-29 23:55:14Z kkolinko $
+ * @version $Id: ClusterDeployer.java 1066569 2011-02-02 18:33:06Z markt $
  */
-import org.apache.catalina.LifecycleException;
+import java.io.File;
 import java.io.IOException;
-import java.net.URL;
+
+import org.apache.catalina.LifecycleException;
 import org.apache.catalina.tribes.ChannelListener;
 
 public interface ClusterDeployer extends ChannelListener {
@@ -46,60 +47,49 @@ public interface ClusterDeployer extends ChannelListener {
      */
     public void stop() throws LifecycleException;
 
-    /**
-     * Sets the deployer for this cluster deployer to use.
-     * @param deployer Deployer
-     */
-    // FIXME
-    //public void setDeployer(Deployer deployer);
-
     /**
      * Install a new web application, whose web application archive is at the
      * specified URL, into this container and all the other
-     * members of the cluster with the specified context path.
-     * A context path of "" (the empty string) should be used for the root
-     * application for this container.  Otherwise, the context path must
-     * start with a slash.
+     * members of the cluster with the specified context name.
      * <p>
      * If this application is successfully installed locally, 
      * a ContainerEvent of type
      * <code>INSTALL_EVENT</code> will be sent to all registered listeners,
      * with the newly created <code>Context</code> as an argument.
      *
-     * @param contextPath The context path to which this application should
+     * @param contextName The context name to which this application should
      *  be installed (must be unique)
-     * @param war A URL of type "jar:" that points to a WAR file, or type
-     *  "file:" that points to an unpacked directory structure containing
-     *  the web application to be installed
+     * @param webapp    A WAR file or unpacked directory structure containing
+     *                  the web application to be installed
      *
-     * @exception IllegalArgumentException if the specified context path
-     *  is malformed (it must be "" or start with a slash)
-     * @exception IllegalStateException if the specified context path
+     * @exception IllegalArgumentException if the specified context name
+     *  is malformed
+     * @exception IllegalStateException if the specified context name
      *  is already attached to an existing web application
      * @exception IOException if an input/output error was encountered
      *  during installation
      */
-    public void install(String contextPath, URL war) throws IOException;
+    public void install(String contextName, File webapp) throws IOException;
 
     /**
      * Remove an existing web application, attached to the specified context
-     * path.  If this application is successfully removed, a
+     * name.  If this application is successfully removed, a
      * ContainerEvent of type <code>REMOVE_EVENT</code> will be sent to all
      * registered listeners, with the removed <code>Context</code> as
      * an argument. Deletes the web application war file and/or directory
      * if they exist in the Host's appBase.
      *
-     * @param contextPath The context path of the application to be removed
+     * @param contextName The context name of the application to be removed
      * @param undeploy boolean flag to remove web application from server
      *
-     * @exception IllegalArgumentException if the specified context path
-     *  is malformed (it must be "" or start with a slash)
-     * @exception IllegalArgumentException if the specified context path does
+     * @exception IllegalArgumentException if the specified context name
+     *  is malformed
+     * @exception IllegalArgumentException if the specified context name does
      *  not identify a currently installed web application
      * @exception IOException if an input/output error occurs during
      *  removal
      */
-    public void remove(String contextPath, boolean undeploy) throws IOException;
+    public void remove(String contextName, boolean undeploy) throws IOException;
 
     /**
      * call from container Background Process
