@@ -18,6 +18,7 @@
 package org.apache.catalina.ha.authenticator;
 
 import java.io.IOException;
+import java.security.Principal;
 import java.util.Map;
 
 import org.apache.catalina.Session;
@@ -90,6 +91,7 @@ public class ClusterSingleSignOnListener extends ClusterListener {
             SingleSignOnMessage msg = (SingleSignOnMessage) myobj;
             int action = msg.getAction();
             Session session = null;
+            Principal principal = null;
 
             if (log.isDebugEnabled())
                 log.debug("SingleSignOnMessage Received with action "
@@ -112,11 +114,17 @@ public class ClusterSingleSignOnListener extends ClusterListener {
                 clusterSSO.deregisterLocal(msg.getSsoId());
                 break;
             case SingleSignOnMessage.REGISTER_SESSION:
-                clusterSSO.registerLocal(msg.getSsoId(), null, msg.getAuthType(),
+                if (msg.getPrincipal() != null) {
+                    principal = msg.getPrincipal().getPrincipal();
+                }
+                clusterSSO.registerLocal(msg.getSsoId(), principal, msg.getAuthType(),
                                          msg.getUsername(), msg.getPassword());
                 break;
             case SingleSignOnMessage.UPDATE_SESSION:
-                clusterSSO.updateLocal(msg.getSsoId(), null, msg.getAuthType(),
+                if (msg.getPrincipal() != null) {
+                    principal = msg.getPrincipal().getPrincipal();
+                }
+                clusterSSO.updateLocal(msg.getSsoId(), principal, msg.getAuthType(),
                                        msg.getUsername(), msg.getPassword());
                 break;
             case SingleSignOnMessage.REMOVE_SESSION:
