@@ -29,15 +29,12 @@ import java.util.ArrayList;
 import javax.naming.Context;
 import javax.sql.DataSource;
 
-import org.apache.naming.ContextBindings;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.ServerFactory;
-import org.apache.catalina.core.StandardServer;
-import org.apache.catalina.util.StringManager;
+import org.apache.naming.ContextBindings;
 
 /**
 *
-* Implmentation of <b>Realm</b> that works with any JDBC JNDI DataSource.
+* Implementation of <b>Realm</b> that works with any JDBC JNDI DataSource.
 * See the JDBCRealm.howto for more details on how to set up the database and
 * for configuration options.
 *
@@ -45,7 +42,7 @@ import org.apache.catalina.util.StringManager;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Revision: 892553 $
+* @version $Revision: 1348499 $
 */
 
 public class DataSourceRealm
@@ -99,14 +96,7 @@ public class DataSourceRealm
 
 
     /**
-     * The string manager for this package.
-     */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
-    /**
-     * The column in the user table that holds the user's credintials
+     * The column in the user table that holds the user's credentials
      */
     protected String userCredCol = null;
 
@@ -280,39 +270,30 @@ public class DataSourceRealm
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      */
+    @Override
     public Principal authenticate(String username, String credentials) {
-    	
-    	// No user or no credentials
-        // Can't possibly authenticate, don't bother the database then
-    	if (username == null || credentials == null) {
-    		return null;
-    	}
         
-    	Connection dbConnection = null;
-
-        try {
-
-            // Ensure that we have an open database connection
-            dbConnection = open();
-            if (dbConnection == null) {
-                // If the db connection open fails, return "not authenticated"
-                return null;
-            }
-            
-            // Acquire a Principal object for this user
-            return authenticate(dbConnection, username, credentials);
-            
-        } catch (SQLException e) {
-            // Log the problem for posterity
-            containerLog.error(sm.getString("dataSourceRealm.exception"), e);
-
-            // Return "not authenticated" for this request
-            return (null);
-
-        } finally {
-        	close(dbConnection);
+        // No user or no credentials
+        // Can't possibly authenticate, don't bother the database then
+        if (username == null || credentials == null) {
+            return null;
         }
+        
+        Connection dbConnection = null;
 
+        // Ensure that we have an open database connection
+        dbConnection = open();
+        if (dbConnection == null) {
+            // If the db connection open fails, return "not authenticated"
+            return null;
+        }
+        
+        // Acquire a Principal object for this user
+        Principal principal = authenticate(dbConnection, username, credentials);
+            
+        close(dbConnection);
+
+        return principal;
     }
 
 
@@ -333,7 +314,7 @@ public class DataSourceRealm
      */
     protected Principal authenticate(Connection dbConnection,
                                                String username,
-                                               String credentials) throws SQLException{
+                                               String credentials) {
 
         String dbCredentials = getPassword(dbConnection, username);
 
@@ -361,7 +342,7 @@ public class DataSourceRealm
         ArrayList<String> list = getRoles(dbConnection, username);
 
         // Create and return a suitable Principal for this user
-        return (new GenericPrincipal(this, username, credentials, list));
+        return (new GenericPrincipal(username, credentials, list));
 
     }
 
@@ -408,12 +389,10 @@ public class DataSourceRealm
                 context = ContextBindings.getClassLoader();
                 context = (Context) context.lookup("comp/env");
             } else {
-                StandardServer server = 
-                    (StandardServer) ServerFactory.getServer();
-                context = server.getGlobalNamingContext();
+                context = getServer().getGlobalNamingContext();
             }
             DataSource dataSource = (DataSource)context.lookup(dataSourceName);
-	    return dataSource.getConnection();
+        return dataSource.getConnection();
         } catch (Exception e) {
             // Log the problem for posterity
             containerLog.error(sm.getString("dataSourceRealm.exception"), e);
@@ -424,6 +403,7 @@ public class DataSourceRealm
     /**
      * Return a short name for this Realm implementation.
      */
+    @Override
     protected String getName() {
 
         return (name);
@@ -433,6 +413,7 @@ public class DataSourceRealm
     /**
      * Return the password associated with the given principal's user name.
      */
+    @Override
     protected String getPassword(String username) {
 
         Connection dbConnection = null;
@@ -444,7 +425,7 @@ public class DataSourceRealm
         }
 
         try {
-        	return getPassword(dbConnection, username);        	
+            return getPassword(dbConnection, username);            
         } finally {
             close(dbConnection);
         }
@@ -456,7 +437,7 @@ public class DataSourceRealm
      * @param username Username for which password should be retrieved
      */
     protected String getPassword(Connection dbConnection, 
-								 String username) {
+                                 String username) {
 
         ResultSet rs = null;
         PreparedStatement stmt = null;
@@ -476,19 +457,19 @@ public class DataSourceRealm
                     sm.getString("dataSourceRealm.getPassword.exception",
                                  username));
         } finally {
-        	try {
-	            if (rs != null) {
-	                rs.close();
-	            }
-	            if (stmt != null) {
-	                stmt.close();
-	            }
-        	} catch (SQLException e) {
+            try {
+                if (rs != null) {
+                    rs.close();
+                }
+                if (stmt != null) {
+                    stmt.close();
+                }
+            } catch (SQLException e) {
                     containerLog.error(
                         sm.getString("dataSourceRealm.getPassword.exception",
-        		             username));
-        		
-        	}
+                             username));
+                
+            }
         }
         
         return null;
@@ -498,18 +479,18 @@ public class DataSourceRealm
     /**
      * Return the Principal associated with the given user name.
      */
+    @Override
     protected Principal getPrincipal(String username) {
-    	Connection dbConnection = open();
+        Connection dbConnection = open();
         if (dbConnection == null) {
-            return new GenericPrincipal(this,username, null, null);
+            return new GenericPrincipal(username, null, null);
         }
         try {
-        	return (new GenericPrincipal(this,
-        			username,
-					getPassword(dbConnection, username),
-					getRoles(dbConnection, username)));
+            return (new GenericPrincipal(username,
+                    getPassword(dbConnection, username),
+                    getRoles(dbConnection, username)));
         } finally {
-        	close(dbConnection);
+            close(dbConnection);
         }
 
     }
@@ -518,7 +499,7 @@ public class DataSourceRealm
      * Return the roles associated with the given user name.
      * @param username Username for which roles should be retrieved
      */
-    protected ArrayList getRoles(String username) {
+    protected ArrayList<String> getRoles(String username) {
 
         Connection dbConnection = null;
 
@@ -531,7 +512,7 @@ public class DataSourceRealm
         try {
             return getRoles(dbConnection, username);
         } finally {
-        	close(dbConnection);
+            close(dbConnection);
         }
     }
     
@@ -542,43 +523,49 @@ public class DataSourceRealm
      */
     protected ArrayList<String> getRoles(Connection dbConnection,
                                      String username) {
-    	
+        
+        if (allRolesMode != AllRolesMode.STRICT_MODE && !isRoleStoreDefined()) {
+            // Using an authentication only configuration and no role store has
+            // been defined so don't spend cycles looking
+            return null;
+        }
+
         ResultSet rs = null;
         PreparedStatement stmt = null;
         ArrayList<String> list = null;
-    	
+        
         try {
-    		stmt = roles(dbConnection, username);
-    		rs = stmt.executeQuery();
-    		list = new ArrayList<String>();
-    		
-    		while (rs.next()) {
-    			String role = rs.getString(1);
-    			if (role != null) {
-    				list.add(role.trim());
-    			}
-    		}
-    		return list;
-    	} catch(SQLException e) {
+            stmt = roles(dbConnection, username);
+            rs = stmt.executeQuery();
+            list = new ArrayList<String>();
+            
+            while (rs.next()) {
+                String role = rs.getString(1);
+                if (role != null) {
+                    list.add(role.trim());
+                }
+            }
+            return list;
+        } catch(SQLException e) {
             containerLog.error(
                 sm.getString("dataSourceRealm.getRoles.exception", username));
         }
-    	finally {
-        	try {
-	            if (rs != null) {
-	                rs.close();
-	            }
-	            if (stmt != null) {
-	                stmt.close();
-	            }
-        	} catch (SQLException e) {
+        finally {
+            try {
+                if (rs != null) {
+                    rs.close();
+                }
+                if (stmt != null) {
+                    stmt.close();
+                }
+            } catch (SQLException e) {
                     containerLog.error(
                         sm.getString("dataSourceRealm.getRoles.exception",
                                      username));
-        	}
+            }
         }
-    	
-    	return null;
+        
+        return null;
     }
 
     /**
@@ -622,23 +609,27 @@ public class DataSourceRealm
 
     }
 
+
+    private boolean isRoleStoreDefined() {
+        return userRoleTable != null || roleNameCol != null;
+    }
+
+
     // ------------------------------------------------------ Lifecycle Methods
 
-
     /**
-     *
-     * Prepare for active use of the public methods of this Component.
+     * Prepare for the beginning of active use of the public methods of this
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
+     *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-
-        // Perform normal superclass initialization
-        super.start();
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         // Create the roles PreparedStatement string
-        StringBuffer temp = new StringBuffer("SELECT ");
+        StringBuilder temp = new StringBuilder("SELECT ");
         temp.append(roleNameCol);
         temp.append(" FROM ");
         temp.append(userRoleTable);
@@ -648,7 +639,7 @@ public class DataSourceRealm
         preparedRoles = temp.toString();
 
         // Create the credentials PreparedStatement string
-        temp = new StringBuffer("SELECT ");
+        temp = new StringBuilder("SELECT ");
         temp.append(userCredCol);
         temp.append(" FROM ");
         temp.append(userTable);
@@ -656,21 +647,7 @@ public class DataSourceRealm
         temp.append(userNameCol);
         temp.append(" = ?");
         preparedCredentials = temp.toString();
+        
+        super.startInternal();
     }
-
-
-    /**
-     * Gracefully shut down active use of the public methods of this Component.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
-     */
-    public void stop() throws LifecycleException {
-
-        // Perform normal superclass finalization
-        super.stop();
-
-    }
-
-
 }
