@@ -22,8 +22,6 @@ package org.apache.catalina.realm;
 import java.io.File;
 import java.io.IOException;
 import java.security.Principal;
-import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.Map;
 
 import javax.security.auth.Subject;
@@ -36,14 +34,9 @@ import javax.security.auth.callback.UnsupportedCallbackException;
 import javax.security.auth.login.FailedLoginException;
 import javax.security.auth.login.LoginException;
 import javax.security.auth.spi.LoginModule;
+import javax.servlet.http.HttpServletRequest;
 
-import org.apache.catalina.Context;
-import org.apache.catalina.Realm;
-import org.apache.catalina.authenticator.Constants;
-import org.apache.catalina.connector.Request;
-import org.apache.catalina.deploy.SecurityConstraint;
-import org.apache.catalina.util.RequestUtil;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.Globals;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.digester.Digester;
@@ -63,7 +56,7 @@ import org.apache.tomcat.util.digester.Digester;
  * <li><strong>debug</strong> - Set to "true" to get debugging messages
  *     generated to System.out.  The default value is <code>false</code>.</li>
  * <li><strong>pathname</strong> - Relative (to the pathname specified by the
- *     "catalina.base" system property) or absolute pahtname to the
+ *     "catalina.base" system property) or absolute pathname to the
  *     XML file containing our user information, in the format supported by
  *     {@link MemoryRealm}.  The default value matches the MemoryRealm
  *     default.</li>
@@ -75,13 +68,13 @@ import org.apache.tomcat.util.digester.Digester;
  * the functionality required of a <code>Realm</code> implementation.</p>
  *
  * @author Craig R. McClanahan
- * @version $Id: JAASMemoryLoginModule.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: JAASMemoryLoginModule.java 1189224 2011-10-26 14:02:40Z kkolinko $
  */
 
-public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, Realm {
+public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule {
     // We need to extend MemoryRealm to avoid class cast
 
-    private static Log log = LogFactory.getLog(JAASMemoryLoginModule.class);
+    private static final Log log = LogFactory.getLog(JAASMemoryLoginModule.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -101,7 +94,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
     /**
      * The configuration information for this <code>LoginModule</code>.
      */
-    protected Map options = null;
+    protected Map<String,?> options = null;
 
 
     /**
@@ -112,27 +105,16 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
 
     /**
      * The <code>Principal</code> identified by our validation, or
-     * <code>null</code> if validation falied.
+     * <code>null</code> if validation failed.
      */
     protected Principal principal = null;
 
 
-    /**
-     * The set of <code>Principals</code> loaded from our configuration file.
-     */
-    protected HashMap principals = new HashMap();
-
-    /**
-     * The string manager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
-
     /**
      * The state information that is shared with other configured
      * <code>LoginModule</code> instances.
      */
-    protected Map sharedState = null;
+    protected Map<String,?> sharedState = null;
 
 
     /**
@@ -158,6 +140,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
      *
      * @exception LoginException if the abort fails
      */
+    @Override
     public boolean abort() throws LoginException {
 
         // If our authentication was not successful, just return false
@@ -188,6 +171,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
      *
      * @exception LoginException if the commit fails
      */
+    @Override
     public boolean commit() throws LoginException {
         log.debug("commit " + principal);
 
@@ -198,7 +182,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
         // Add our Principal to the Subject if needed
         if (!subject.getPrincipals().contains(principal)) {
             subject.getPrincipals().add(principal);
-            // Add the roles as additional sudjucts as per the contract with the
+            // Add the roles as additional subjects as per the contract with the
             // JAASRealm
             if (principal instanceof GenericPrincipal) {
                 String roles[] = ((GenericPrincipal) principal).getRoles();
@@ -216,55 +200,6 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
     }
 
     
-    /**
-     * Return the SecurityConstraints configured to guard the request URI for
-     * this request, or <code>null</code> if there is no such constraint.
-     *
-     * @param request Request we are processing
-     * @param context Context the Request is mapped to
-     */
-    public SecurityConstraint [] findSecurityConstraints(Request request,
-                                                     Context context) {
-        ArrayList<SecurityConstraint> results = null;
-        // Are there any defined security constraints?
-        SecurityConstraint constraints[] = context.findConstraints();
-        if ((constraints == null) || (constraints.length == 0)) {
-            if (context.getLogger().isDebugEnabled())
-                context.getLogger().debug("  No applicable constraints defined");
-            return (null);
-        }
-
-        // Check each defined security constraint
-        String uri = request.getDecodedRequestURI();
-        String contextPath = request.getContextPath();
-        if (contextPath.length() > 0)
-            uri = uri.substring(contextPath.length());
-        uri = RequestUtil.URLDecode(uri); // Before checking constraints
-        String method = request.getMethod();
-        for (int i = 0; i < constraints.length; i++) {
-            if (context.getLogger().isDebugEnabled())
-                context.getLogger().debug("  Checking constraint '" + constraints[i] +
-                    "' against " + method + " " + uri + " --> " +
-                    constraints[i].included(uri, method));
-            if (constraints[i].included(uri, method)) {
-                if(results == null) {
-                    results = new ArrayList<SecurityConstraint>();
-                }
-                results.add(constraints[i]);
-            }
-        }
-
-        // No applicable security constraint was found
-        if (context.getLogger().isDebugEnabled())
-            context.getLogger().debug("  No applicable constraint located");
-        if(results == null)
-            return null;
-        SecurityConstraint [] array = new SecurityConstraint[results.size()];
-        System.arraycopy(results.toArray(), 0, array, 0, array.length);
-        return array;
-    }
-    
-    
     /**
      * Initialize this <code>LoginModule</code> with the specified
      * configuration information.
@@ -277,8 +212,9 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
      * @param options Configuration information for this specific
      *  <code>LoginModule</code> instance
      */
+    @Override
     public void initialize(Subject subject, CallbackHandler callbackHandler,
-                           Map sharedState, Map options) {
+                           Map<String,?> sharedState, Map<String,?> options) {
         log.debug("Init");
 
         // Save configuration values
@@ -306,6 +242,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
      *
      * @exception LoginException if the authentication fails
      */
+    @Override
     public boolean login() throws LoginException {
 
         // Set up our CallbackHandler requests
@@ -355,10 +292,10 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
         if (authMethod == null) {
             // BASIC or FORM
             principal = super.authenticate(username, password);
-        } else if (authMethod.equals(Constants.DIGEST_METHOD)) {
+        } else if (authMethod.equals(HttpServletRequest.DIGEST_AUTH)) {
             principal = super.authenticate(username, password, nonce, nc,
                     cnonce, qop, realmName, md5a2);
-        } else if (authMethod.equals(Constants.CERT_METHOD)) {
+        } else if (authMethod.equals(HttpServletRequest.CLIENT_CERT_AUTH)) {
             principal = super.getPrincipal(username);
         } else {
             throw new LoginException("Unknown authentication method");
@@ -380,11 +317,12 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
     /**
      * Log out this user.
      *
-     * @return <code>true</code> in all cases because thie
+     * @return <code>true</code> in all cases because the
      *  <code>LoginModule</code> should not be ignored
      *
      * @exception LoginException if logging out failed
      */
+    @Override
     public boolean logout() throws LoginException {
 
         subject.getPrincipals().remove(principal);
@@ -407,7 +345,7 @@ public class JAASMemoryLoginModule extends MemoryRealm implements LoginModule, R
         // Validate the existence of our configuration file
         File file = new File(pathname);
         if (!file.isAbsolute())
-            file = new File(System.getProperty("catalina.base"), pathname);
+            file = new File(System.getProperty(Globals.CATALINA_BASE_PROP), pathname);
         if (!file.exists() || !file.canRead()) {
             log.warn("Cannot load configuration file " + file.getAbsolutePath());
             return;
