@@ -31,17 +31,17 @@ import javax.security.auth.login.CredentialExpiredException;
 import javax.security.auth.login.FailedLoginException;
 import javax.security.auth.login.LoginContext;
 import javax.security.auth.login.LoginException;
+import javax.servlet.http.HttpServletRequest;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.authenticator.Constants;
-import org.apache.catalina.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
- * <p>Implmentation of <b>Realm</b> that authenticates users via the <em>Java
+ * <p>Implementation of <b>Realm</b> that authenticates users via the <em>Java
  * Authentication and Authorization Service</em> (JAAS).  JAAS support requires
  * either JDK 1.4 (which includes it as part of the standard platform) or
  * JDK 1.3 (with the plug-in <code>jaas.jar</code> file).</p>
@@ -82,7 +82,7 @@ import org.apache.juli.logging.LogFactory;
  *     <code>Subject.getPrincipals()</code>, it will identify the first
  *     <code>Principal</code> that matches the "user classes" list as the
  *     <code>Principal</code> for this user.</li>
- * <li>As this Realm iterates over the <code>Princpals</code> returned by
+ * <li>As this Realm iterates over the <code>Principals</code> returned by
  *     <code>Subject.getPrincipals()</code>, it will accumulate the set of
  *     all <code>Principals</code> matching the "role classes" list as
  *     identifying the security roles for this user.</li>
@@ -122,13 +122,13 @@ org.foobar.auth.DatabaseLoginModule REQUIRED
 *
 * @author Craig R. McClanahan
 * @author Yoav Shapira
- * @version $Id: JAASRealm.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: JAASRealm.java 1189224 2011-10-26 14:02:40Z kkolinko $
  */
 
 public class JAASRealm
     extends RealmBase
  {
-    private static Log log = LogFactory.getLog(JAASRealm.class);
+    private static final Log log = LogFactory.getLog(JAASRealm.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -159,13 +159,6 @@ public class JAASRealm
     protected List<String> roleClasses = new ArrayList<String>();
 
 
-    /**
-     * The string manager for this package.
-     */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
     /**
      * The set of user class names, split out for easy processing.
      */
@@ -187,6 +180,7 @@ public class JAASRealm
      * setter for the <code>appName</code> member variable
      * @deprecated JAAS should use the <code>Engine</code> (domain) name and webpp/host overrides
      */
+    @Deprecated
     public void setAppName(String name) {
         appName = name;
     }
@@ -216,14 +210,18 @@ public class JAASRealm
      * @return The value of useContextClassLoader
      */
     public boolean isUseContextClassLoader() {
-	return useContextClassLoader;
+        return useContextClassLoader;
     } 
 
+    @Override
     public void setContainer(Container container) {
         super.setContainer(container);
 
         if( appName==null  ) {
-            String name=container.getName();
+            String name = container.getName();
+            if (!name.startsWith("/")) {
+                name = "/" + name;
+            }
             name = makeLegalForJAAS(name);
 
             appName=name;
@@ -273,7 +271,7 @@ public class JAASRealm
          for (int i=0; i<classNames.length; i++) {
              if (classNames[i].length()==0) continue;        
              try {
-                 Class principalClass = Class.forName(classNames[i], false,
+                 Class<?> principalClass = Class.forName(classNames[i], false,
                          loader);
                  if (Principal.class.isAssignableFrom(principalClass)) {
                      classNamesList.add(classNames[i]);
@@ -331,6 +329,7 @@ public class JAASRealm
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      */
+    @Override
     public Principal authenticate(String username, String credentials) {
         return authenticate(username,
                 new JAASCallbackHandler(this, username, credentials));
@@ -346,18 +345,19 @@ public class JAASRealm
      * @param nonce         Server generated nonce
      * @param nc            Nonce count
      * @param cnonce        Client generated nonce
-     * @param qop           Quality of protection aplied to the message
+     * @param qop           Quality of protection applied to the message
      * @param realmName     Realm name
      * @param md5a2         Second MD5 digest used to calculate the digest
      *                          MD5(Method + ":" + uri)
      */
+    @Override
     public Principal authenticate(String username, String clientDigest,
             String nonce, String nc, String cnonce, String qop,
             String realmName, String md5a2) {
         return authenticate(username,
                 new JAASCallbackHandler(this, username, clientDigest, nonce,
                         nc, cnonce, qop, realmName, md5a2,
-                        Constants.DIGEST_METHOD));
+                        HttpServletRequest.DIGEST_AUTH));
     }
 
 
@@ -393,6 +393,7 @@ public class JAASRealm
         try {
             loginContext = new LoginContext(appName, callbackHandler);
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             log.error(sm.getString("jaasRealm.unexpectedError"), e);
             return (null);
         } finally {
@@ -430,6 +431,7 @@ public class JAASRealm
             log.warn(sm.getString("jaasRealm.loginException", username), e);
             return (null);
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             log.error(sm.getString("jaasRealm.unexpectedError"), e);
             return (null);
         }
@@ -457,6 +459,7 @@ public class JAASRealm
     /**
      * Return a short name for this <code>Realm</code> implementation.
      */
+    @Override
     protected String getName() {
 
         return (name);
@@ -469,6 +472,7 @@ public class JAASRealm
      * always returns null as the JAASRealm has no way of obtaining this
      * information.
      */
+    @Override
     protected String getPassword(String username) {
 
         return (null);
@@ -479,20 +483,32 @@ public class JAASRealm
     /**
      * Return the <code>Principal</code> associated with the given user name.
      */
+    @Override
     protected Principal getPrincipal(String username) {
 
         return authenticate(username,
                 new JAASCallbackHandler(this, username, null, null, null, null,
-                        null, null, null, Constants.CERT_METHOD));
+                        null, null, null, HttpServletRequest.CLIENT_CERT_AUTH));
 
     }
 
 
     /**
-     * @deprecated
-     * Use {@link JAASRealm#createPrincipal(String, Subject, LoginContext)}
+     * Identify and return a <code>java.security.Principal</code> instance
+     * representing the authenticated user for the specified <code>Subject</code>.
+     * The Principal is constructed by scanning the list of Principals returned
+     * by the JAASLoginModule. The first <code>Principal</code> object that matches
+     * one of the class names supplied as a "user class" is the user Principal.
+     * This object is returned to the caller.
+     * Any remaining principal objects returned by the LoginModules are mapped to  
+     * roles, but only if their respective classes match one of the "role class" classes. 
+     * If a user Principal cannot be constructed, return <code>null</code>.
+     * @param subject The <code>Subject</code> representing the logged-in user
+     * @param loginContext Associated with the Principal so
+     *                     {@link LoginContext#logout()} can be called later
      */
-    protected Principal createPrincipal(String username, Subject subject) {
+    protected Principal createPrincipal(String username, Subject subject,
+            LoginContext loginContext) {
         // Prepare to scan the Principals for this Subject
 
         List<String> roles = new ArrayList<String>();
@@ -539,30 +555,8 @@ public class JAASRealm
         }
 
         // Return the resulting Principal for our authenticated user
-        return new GenericPrincipal(this, username, null, roles, userPrincipal);
-    }
-
-    /**
-     * Identify and return a <code>java.security.Principal</code> instance
-     * representing the authenticated user for the specified <code>Subject</code>.
-     * The Principal is constructed by scanning the list of Principals returned
-     * by the JAASLoginModule. The first <code>Principal</code> object that matches
-     * one of the class names supplied as a "user class" is the user Principal.
-     * This object is returned to the caller.
-     * Any remaining principal objects returned by the LoginModules are mapped to  
-     * roles, but only if their respective classes match one of the "role class" classes. 
-     * If a user Principal cannot be constructed, return <code>null</code>.
-     * @param subject The <code>Subject</code> representing the logged-in user
-     * @param loginContext Associated with the Principal so
-     *                     {@link LoginContext#logout()} can be called later
-     */
-    protected Principal createPrincipal(String username, Subject subject,
-            LoginContext loginContext) {
-        Principal principal = createPrincipal(username, subject);
-        if (principal instanceof GenericPrincipal) {
-            ((GenericPrincipal) principal).setLoginContext(loginContext);
-        }
-        return principal;
+        return new GenericPrincipal(username, null, roles, userPrincipal,
+                loginContext);
     }
 
      /**
@@ -595,37 +589,22 @@ public class JAASRealm
     // ------------------------------------------------------ Lifecycle Methods
 
 
-    /**
-     *
-     * Prepare for active use of the public methods of this <code>Component</code>.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
-     */
-    public void start() throws LifecycleException {
-
-        // Perform normal superclass initialization
-        super.start();
+     /**
+      * Prepare for the beginning of active use of the public methods of this
+      * component and implement the requirements of
+      * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
+      *
+      * @exception LifecycleException if this component detects a fatal error
+      *  that prevents this component from being used
+      */
+     @Override
+     protected void startInternal() throws LifecycleException {
 
         // These need to be called after loading configuration, in case
         // useContextClassLoader appears after them in xml config
         parseClassNames(userClassNames, userClasses);
         parseClassNames(roleClassNames, roleClasses);
-    }
-
-
-    /**
-     * Gracefully shut down active use of the public methods of this <code>Component</code>.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
-     */
-    public void stop() throws LifecycleException {
-
-        // Perform normal superclass finalization
-        super.stop();
-
-    }
-
 
+        super.startInternal();
+     }
 }
