@@ -29,12 +29,12 @@ import java.util.ArrayList;
 import java.util.Properties;
 
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
 *
-* Implmentation of <b>Realm</b> that works with any JDBC supported database.
+* Implementation of <b>Realm</b> that works with any JDBC supported database.
 * See the JDBCRealm.howto for more details on how to set up the database and
 * for configuration options.
 *
@@ -47,7 +47,7 @@ import org.apache.catalina.util.StringManager;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Revision: 892553 $ $Date: 2009-12-20 03:32:53 +0100 (So, 20. Dez 2009) $
+* @version $Id: JDBCRealm.java 1348499 2012-06-09 20:37:14Z markt $
 */
 
 public class JDBCRealm
@@ -126,14 +126,7 @@ public class JDBCRealm
 
 
     /**
-     * The string manager for this package.
-     */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
-    /**
-     * The column in the user table that holds the user's credintials
+     * The column in the user table that holds the user's credentials
      */
     protected String userCredCol = null;
 
@@ -340,11 +333,12 @@ public class JDBCRealm
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      */
+    @Override
     public synchronized Principal authenticate(String username, String credentials) {
 
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
-        // This needs rewritten wuth better pooling support, the existing code
+        // This needs rewritten with better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
         // with the connections.
         // The code below will try twice if there is a SQLException so the
@@ -436,7 +430,7 @@ public class JDBCRealm
         ArrayList<String> roles = getRoles(username);
         
         // Create and return a suitable Principal for this user
-        return (new GenericPrincipal(this, username, credentials, roles));
+        return (new GenericPrincipal(username, credentials, roles));
 
     }
 
@@ -456,7 +450,7 @@ public class JDBCRealm
         try {
             preparedCredentials.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedCredentials = null;
 
@@ -464,7 +458,7 @@ public class JDBCRealm
         try {
             preparedRoles.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedRoles = null;
 
@@ -495,7 +489,7 @@ public class JDBCRealm
         throws SQLException {
 
         if (preparedCredentials == null) {
-            StringBuffer sb = new StringBuffer("SELECT ");
+            StringBuilder sb = new StringBuilder("SELECT ");
             sb.append(userCredCol);
             sb.append(" FROM ");
             sb.append(userTable);
@@ -524,6 +518,7 @@ public class JDBCRealm
     /**
      * Return a short name for this Realm implementation.
      */
+    @Override
     protected String getName() {
 
         return (name);
@@ -534,6 +529,7 @@ public class JDBCRealm
     /**
      * Return the password associated with the given principal's user name.
      */
+    @Override
     protected synchronized String getPassword(String username) {
 
         // Look up the user's credentials
@@ -541,9 +537,9 @@ public class JDBCRealm
         PreparedStatement stmt = null;
         ResultSet rs = null;
 
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
-        // This needs rewritten wuth better pooling support, the existing code
+        // This needs rewritten with better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
         // with the connections.
         // The code below will try twice if there is a SQLException so the
@@ -604,10 +600,10 @@ public class JDBCRealm
     /**
      * Return the Principal associated with the given user name.
      */
+    @Override
     protected synchronized Principal getPrincipal(String username) {
 
-        return (new GenericPrincipal(this,
-                                     username,
+        return (new GenericPrincipal(username,
                                      getPassword(username),
                                      getRoles(username)));
 
@@ -619,10 +615,16 @@ public class JDBCRealm
      */
     protected ArrayList<String> getRoles(String username) {
         
+        if (allRolesMode != AllRolesMode.STRICT_MODE && !isRoleStoreDefined()) {
+            // Using an authentication only configuration and no role store has
+            // been defined so don't spend cycles looking
+            return null;
+        }
+
         PreparedStatement stmt = null;
         ResultSet rs = null;
 
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
         // This needs rewritten wuth better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
@@ -678,8 +680,7 @@ public class JDBCRealm
             numberOfTries--;
         }
         
-        return (null);
-        
+        return null;
     }
     
     
@@ -698,10 +699,11 @@ public class JDBCRealm
         // Instantiate our database driver if necessary
         if (driver == null) {
             try {
-                Class clazz = Class.forName(driverName);
+                Class<?> clazz = Class.forName(driverName);
                 driver = (Driver) clazz.newInstance();
             } catch (Throwable e) {
-                throw new SQLException(e.getMessage());
+                ExceptionUtils.handleThrowable(e);
+                throw new SQLException(e.getMessage(), e);
             }
         }
 
@@ -729,7 +731,7 @@ public class JDBCRealm
      */
     protected void release(Connection dbConnection) {
 
-        ; // NO-OP since we are not pooling anything
+        // NO-OP since we are not pooling anything
 
     }
 
@@ -748,7 +750,7 @@ public class JDBCRealm
         throws SQLException {
 
         if (preparedRoles == null) {
-            StringBuffer sb = new StringBuffer("SELECT ");
+            StringBuilder sb = new StringBuilder("SELECT ");
             sb.append(roleNameCol);
             sb.append(" FROM ");
             sb.append(userRoleTable);
@@ -765,20 +767,23 @@ public class JDBCRealm
     }
 
 
+    private boolean isRoleStoreDefined() {
+        return userRoleTable != null || roleNameCol != null;
+    }
+
+
     // ------------------------------------------------------ Lifecycle Methods
 
-
     /**
-     *
-     * Prepare for active use of the public methods of this Component.
+     * Prepare for the beginning of active use of the public methods of this
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
+     *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-
-        // Perform normal superclass initialization
-        super.start();
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         // Validate that we can open our connection - but let tomcat
         // startup in case the database is temporarily unavailable
@@ -788,19 +793,22 @@ public class JDBCRealm
             containerLog.error(sm.getString("jdbcRealm.open"), e);
         }
 
+        super.startInternal();
     }
 
 
     /**
-     * Gracefully shut down active use of the public methods of this Component.
+     * Gracefully terminate the active use of the public methods of this
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that needs to be reported
      */
-    public void stop() throws LifecycleException {
+     @Override
+    protected void stopInternal() throws LifecycleException {
 
-        // Perform normal superclass finalization
-        super.stop();
+        super.stopInternal();
 
         // Close any open DB connection
         close(this.dbConnection);
