@@ -18,8 +18,6 @@
 
 package org.apache.catalina.session;
 
-import java.beans.PropertyChangeEvent;
-import java.beans.PropertyChangeListener;
 import java.io.IOException;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
@@ -27,22 +25,18 @@ import java.security.PrivilegedExceptionAction;
 import java.util.HashMap;
 import java.util.Map;
 
-import org.apache.catalina.Container;
-import org.apache.catalina.Context;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Session;
 import org.apache.catalina.Store;
-import org.apache.catalina.util.LifecycleSupport;
-
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 /**
  * Extends the <b>ManagerBase</b> class to implement most of the
  * functionality required by a Manager which supports any kind of
- * persistence, even if onlyfor  restarts.
+ * persistence, even if only for  restarts.
  * <p>
  * <b>IMPLEMENTATION NOTE</b>:  Correct behavior of session storing and
  * reloading depends upon external calls to the <code>start()</code> and
@@ -50,31 +44,31 @@ import org.apache.juli.logging.LogFactory;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Id: PersistentManagerBase.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: PersistentManagerBase.java 1058571 2011-01-13 14:08:24Z markt $
  */
 
-public abstract class PersistentManagerBase
-    extends ManagerBase
-    implements Lifecycle, PropertyChangeListener {
+public abstract class PersistentManagerBase extends ManagerBase {
 
-    private static Log log = LogFactory.getLog(PersistentManagerBase.class);
+    private static final Log log = LogFactory.getLog(PersistentManagerBase.class);
 
     // ---------------------------------------------------- Security Classes
 
     private class PrivilegedStoreClear
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Void> {
 
-        PrivilegedStoreClear() {            
+        PrivilegedStoreClear() {
+            // NOOP
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Void run() throws Exception{
            store.clear();
            return null;
         }                       
     }   
      
     private class PrivilegedStoreRemove
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Void> {
 
         private String id;    
             
@@ -82,14 +76,15 @@ public abstract class PersistentManagerBase
             this.id = id;
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Void run() throws Exception{
            store.remove(id);
            return null;
         }                       
     }   
      
     private class PrivilegedStoreLoad
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Session> {
 
         private String id;    
             
@@ -97,13 +92,14 @@ public abstract class PersistentManagerBase
             this.id = id;
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Session run() throws Exception{
            return store.load(id);
         }                       
     }   
           
     private class PrivilegedStoreSave
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Void> {
 
         private Session session;    
             
@@ -111,19 +107,22 @@ public abstract class PersistentManagerBase
             this.session = session;
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Void run() throws Exception{
            store.save(session);
            return null;
         }                       
     }   
      
     private class PrivilegedStoreKeys
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<String[]> {
 
-        PrivilegedStoreKeys() {     
+        PrivilegedStoreKeys() {
+            // NOOP
         }
 
-        public Object run() throws Exception{
+        @Override
+        public String[] run() throws Exception{
            return store.keys();
         }                       
     }
@@ -137,30 +136,12 @@ public abstract class PersistentManagerBase
     private static final String info = "PersistentManagerBase/1.1";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
-    /**
-     * The maximum number of active Sessions allowed, or -1 for no limit.
-     */
-    protected int maxActiveSessions = -1;
-
-
     /**
      * The descriptive name of this Manager implementation (for logging).
      */
     private static String name = "PersistentManagerBase";
 
 
-    /**
-     * Has this component been started yet?
-     */
-    protected boolean started = false;
-
-
     /**
      * Store object which will manage the Session store.
      */
@@ -196,36 +177,21 @@ public abstract class PersistentManagerBase
     protected int maxIdleSwap = -1;
 
 
-    /**
-     * Number of session creations that failed due to maxActiveSessions.
-     */
-    protected int rejectedSessions = 0;
-
-
-    /**
-     * Processing time during session expiration and passivation.
-     */
-    protected long processingTime = 0;
-
-
     /**
      * Sessions currently being swapped in and the associated locks
      */
     private final Map<String,Object> sessionSwapInLocks =
-    	new HashMap<String,Object>();
+        new HashMap<String,Object>();
 
 
     // ------------------------------------------------------------- Properties
 
-    
-  
-
 
     /**
-	 * Indicates how many seconds old a session can get, after its last use in a
-	 * request, before it should be backed up to the store. -1 means sessions
-	 * are not backed up.
-	 */
+     * Indicates how many seconds old a session can get, after its last use in a
+     * request, before it should be backed up to the store. -1 means sessions
+     * are not backed up.
+     */
     public int getMaxIdleBackup() {
 
         return maxIdleBackup;
@@ -260,8 +226,8 @@ public abstract class PersistentManagerBase
         int oldBackup = this.maxIdleBackup;
         this.maxIdleBackup = backup;
         support.firePropertyChange("maxIdleBackup",
-                                   new Integer(oldBackup),
-                                   new Integer(this.maxIdleBackup));
+                                   Integer.valueOf(oldBackup),
+                                   Integer.valueOf(this.maxIdleBackup));
 
     }
 
@@ -288,8 +254,8 @@ public abstract class PersistentManagerBase
         int oldMaxIdleSwap = this.maxIdleSwap;
         this.maxIdleSwap = max;
         support.firePropertyChange("maxIdleSwap",
-                                   new Integer(oldMaxIdleSwap),
-                                   new Integer(this.maxIdleSwap));
+                                   Integer.valueOf(oldMaxIdleSwap),
+                                   Integer.valueOf(this.maxIdleSwap));
 
     }
 
@@ -318,35 +284,8 @@ public abstract class PersistentManagerBase
         int oldMinIdleSwap = this.minIdleSwap;
         this.minIdleSwap = min;
         support.firePropertyChange("minIdleSwap",
-                                   new Integer(oldMinIdleSwap),
-                                   new Integer(this.minIdleSwap));
-
-    }
-
-
-    /**
-	 * Set the Container with which this Manager has been associated. If it is a
-	 * Context (the usual case), listen for changes to the session timeout
-	 * property.
-	 * 
-	 * @param container
-	 *            The associated Container
-	 */
-    public void setContainer(Container container) {
-
-        // De-register from the old Container (if any)
-        if ((this.container != null) && (this.container instanceof Context))
-            ((Context) this.container).removePropertyChangeListener(this);
-
-        // Default processing provided by our superclass
-        super.setContainer(container);
-
-        // Register with the new Container (if any)
-        if ((this.container != null) && (this.container instanceof Context)) {
-            setMaxInactiveInterval
-                ( ((Context) this.container).getSessionTimeout()*60 );
-            ((Context) this.container).addPropertyChangeListener(this);
-        }
+                                   Integer.valueOf(oldMinIdleSwap),
+                                   Integer.valueOf(this.minIdleSwap));
 
     }
 
@@ -356,6 +295,7 @@ public abstract class PersistentManagerBase
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -380,51 +320,10 @@ public abstract class PersistentManagerBase
     }
 
 
-    /**
-     * Return the maximum number of active Sessions allowed, or -1 for
-     * no limit.
-     */
-    public int getMaxActiveSessions() {
-
-        return (this.maxActiveSessions);
-
-    }
-
-
-    /**
-     * Set the maximum number of actives Sessions allowed, or -1 for
-     * no limit.
-     *
-     * @param max The new maximum number of sessions
-     */
-    public void setMaxActiveSessions(int max) {
-
-        int oldMaxActiveSessions = this.maxActiveSessions;
-        this.maxActiveSessions = max;
-        support.firePropertyChange("maxActiveSessions",
-                                   new Integer(oldMaxActiveSessions),
-                                   new Integer(this.maxActiveSessions));
-
-    }
-
-
-    /** 
-     * Number of session creations that failed due to maxActiveSessions.
-     *
-     * @return The count
-     */
-    public int getRejectedSessions() {
-        return rejectedSessions;
-    }
-
-    
-    public void setRejectedSessions(int rejectedSessions) {
-        this.rejectedSessions = rejectedSessions;
-    }
-
     /**
      * Return the descriptive short name of this Manager implementation.
      */
+    @Override
     public String getName() {
 
         return (name);
@@ -432,26 +331,6 @@ public abstract class PersistentManagerBase
     }
 
 
-    /**
-     * Get the started status.
-     */
-    protected boolean isStarted() {
-
-        return started;
-
-    }
-
-
-    /**
-     * Set the started flag
-     */
-    protected void setStarted(boolean started) {
-
-        this.started = started;
-
-    }
-
-
     /**
      * Set the Store object which will manage persistent Session
      * storage for this Manager.
@@ -476,7 +355,6 @@ public abstract class PersistentManagerBase
     }
 
 
-
     /**
      * Indicates whether sessions are saved when the Manager is shut down
      * properly. This requires the unload() method to be called.
@@ -505,8 +383,8 @@ public abstract class PersistentManagerBase
         boolean oldSaveOnRestart = this.saveOnRestart;
         this.saveOnRestart = saveOnRestart;
         support.firePropertyChange("saveOnRestart",
-                                   new Boolean(oldSaveOnRestart),
-                                   new Boolean(this.saveOnRestart));
+                                   Boolean.valueOf(oldSaveOnRestart),
+                                   Boolean.valueOf(this.saveOnRestart));
 
     }
 
@@ -528,15 +406,14 @@ public abstract class PersistentManagerBase
                     AccessController.doPrivileged(new PrivilegedStoreClear());
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
-                    log.error("Exception clearing the Store: " + exception);
-                    exception.printStackTrace();                        
+                    log.error("Exception clearing the Store: " + exception,
+                            exception);
                 }
             } else {
                 store.clear();
             }
         } catch (IOException e) {
-            log.error("Exception clearing the Store: " + e);
-            e.printStackTrace();
+            log.error("Exception clearing the Store: " + e, e);
         }
 
     }
@@ -545,8 +422,9 @@ public abstract class PersistentManagerBase
     /**
      * Implements the Manager interface, direct call to processExpires and processPersistenceChecks
      */
-	public void processExpires() {
-		
+    @Override
+    public void processExpires() {
+        
         long timeNow = System.currentTimeMillis();
         Session sessions[] = findSessions();
         int expireHere = 0 ;
@@ -554,7 +432,7 @@ public abstract class PersistentManagerBase
              log.debug("Start expire sessions " + getName() + " at " + timeNow + " sessioncount " + sessions.length);
         for (int i = 0; i < sessions.length; i++) {
             if (!sessions[i].isValid()) {
-                expiredSessions++;
+                expiredSessions.incrementAndGet();
                 expireHere++;
             }
         }
@@ -567,8 +445,8 @@ public abstract class PersistentManagerBase
         if(log.isDebugEnabled())
              log.debug("End expire sessions " + getName() + " processingTime " + (timeEnd - timeNow) + " expired sessions: " + expireHere);
         processingTime += (timeEnd - timeNow);
- 		
-	}
+         
+    }
 
 
     /**
@@ -597,6 +475,7 @@ public abstract class PersistentManagerBase
      * @exception IOException if an input/output error occurs while
      *  processing this request
      */
+    @Override
     public Session findSession(String id) throws IOException {
 
         Session session = super.findSession(id);
@@ -646,6 +525,7 @@ public abstract class PersistentManagerBase
      * class. In order to use it, a subclass must specifically call it,
      * for example in the start() and/or processPersistenceChecks() methods.
      */
+    @Override
     public void load() {
 
         // Initialize our internal data structures
@@ -658,13 +538,13 @@ public abstract class PersistentManagerBase
         try {
             if (SecurityUtil.isPackageProtectionEnabled()){
                 try{
-                    ids = (String[])
-                        AccessController.doPrivileged(new PrivilegedStoreKeys());
+                    ids = AccessController.doPrivileged(
+                            new PrivilegedStoreKeys());
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
                     log.error("Exception in the Store during load: "
-                              + exception);
-                    exception.printStackTrace();                        
+                              + exception, exception);
+                    return;
                 }
             } else {
                 ids = store.keys();
@@ -697,9 +577,10 @@ public abstract class PersistentManagerBase
      *
      * @param session Session to be removed
      */
-    public void remove(Session session) {
+    @Override
+    public void remove(Session session, boolean update) {
 
-        super.remove (session);
+        super.remove (session, update);
 
         if (store != null){
             removeSession(session.getIdInternal());
@@ -721,15 +602,13 @@ public abstract class PersistentManagerBase
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
                     log.error("Exception in the Store during removeSession: "
-                              + exception);
-                    exception.printStackTrace();                        
+                              + exception, exception);
                 }
             } else {
                  store.remove(id);
             }               
         } catch (IOException e) {
-            log.error("Exception removing session  " + e.getMessage());
-            e.printStackTrace();
+            log.error("Exception removing session  " + e.getMessage(), e);
         }        
     }
 
@@ -742,6 +621,7 @@ public abstract class PersistentManagerBase
      * class. In order to use it, a subclass must specifically call it,
      * for example in the stop() and/or processPersistenceChecks() methods.
      */
+    @Override
     public void unload() {
 
         if (store == null)
@@ -760,7 +640,7 @@ public abstract class PersistentManagerBase
             try {
                 swapOut(sessions[i]);
             } catch (IOException e) {
-                ;   // This is logged in writeSession()
+                // This is logged in writeSession()
             }
 
     }
@@ -810,7 +690,7 @@ public abstract class PersistentManagerBase
                 try {
                     if (SecurityUtil.isPackageProtectionEnabled()){
                         try {
-                            session = (Session) AccessController.doPrivileged(
+                            session = AccessController.doPrivileged(
                                     new PrivilegedStoreLoad(id));
                         } catch (PrivilegedActionException ex) {
                             Exception e = ex.getException();
@@ -885,7 +765,7 @@ public abstract class PersistentManagerBase
 
         ((StandardSession)session).passivate();
         writeSession(session);
-        super.remove(session);
+        super.remove(session, true);
         session.recycle();
 
     }
@@ -908,9 +788,11 @@ public abstract class PersistentManagerBase
                     AccessController.doPrivileged(new PrivilegedStoreSave(session));
                 }catch(PrivilegedActionException ex){
                     Exception exception = ex.getException();
+                    if (exception instanceof IOException) {
+                        throw (IOException) exception;
+                    }
                     log.error("Exception in the Store during writeSession: "
-                              + exception);
-                    exception.printStackTrace();                        
+                              + exception, exception);
                 }
             } else {
                  store.save(session);
@@ -924,102 +806,42 @@ public abstract class PersistentManagerBase
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
     /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
 
-        // Validate and update our current component state
-        if (started) {
-            log.info(sm.getString("standardManager.alreadyStarted"));
-            return;
-        }
-        if( ! initialized )
-            init();
-        
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
-
-        // Force initialization of the random number generator
-        if (log.isDebugEnabled())
-            log.debug("Force random number initialization starting");
-        String dummy = generateSessionId();
-        if (log.isDebugEnabled())
-            log.debug("Force random number initialization completed");
+        super.startInternal();
 
         if (store == null)
             log.error("No Store configured, persistence disabled");
         else if (store instanceof Lifecycle)
             ((Lifecycle)store).start();
 
+        setState(LifecycleState.STARTING);
     }
 
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
+     *  that prevents this component from being used
      */
-   public void stop() throws LifecycleException {
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
 
         if (log.isDebugEnabled())
             log.debug("Stopping");
 
-        // Validate and update our current component state
-        if (!isStarted()) {
-            log.info(sm.getString("standardManager.notStarted"));
-            return;
-        }
+        setState(LifecycleState.STOPPING);
         
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        setStarted(false);
-
         if (getStore() != null && saveOnRestart) {
             unload();
         } else {
@@ -1037,40 +859,7 @@ public abstract class PersistentManagerBase
             ((Lifecycle)getStore()).stop();
 
         // Require a new random number generator if we are restarted
-        this.random = null;
-
-        if( initialized )
-            destroy();
-
-    }
-
-
-    // ----------------------------------------- PropertyChangeListener Methods
-
-
-    /**
-     * Process property change events from our associated Context.
-     *
-     * @param event The property change event that has occurred
-     */
-    public void propertyChange(PropertyChangeEvent event) {
-
-        // Validate the source of this event
-        if (!(event.getSource() instanceof Context))
-            return;
-        Context context = (Context) event.getSource();
-
-        // Process a relevant property change
-        if (event.getPropertyName().equals("sessionTimeout")) {
-            try {
-                setMaxInactiveInterval
-                    ( ((Integer) event.getNewValue()).intValue()*60 );
-            } catch (NumberFormatException e) {
-                log.error(sm.getString("standardManager.sessionTimeout",
-                                 event.getNewValue().toString()));
-            }
-        }
-
+        super.stopInternal();
     }
 
 
@@ -1082,7 +871,7 @@ public abstract class PersistentManagerBase
      */
     protected void processMaxIdleSwaps() {
 
-        if (!isStarted() || maxIdleSwap < 0)
+        if (!getState().isAvailable() || maxIdleSwap < 0)
             return;
 
         Session sessions[] = findSessions();
@@ -1096,21 +885,22 @@ public abstract class PersistentManagerBase
                     if (!session.isValid())
                         continue;
                     int timeIdle = // Truncate, do not round up
-                        (int) ((timeNow - session.getLastAccessedTime()) / 1000L);
+                        (int) ((timeNow - session.getThisAccessedTime()) / 1000L);
                     if (timeIdle > maxIdleSwap && timeIdle > minIdleSwap) {
                         if (session.accessCount != null &&
                                 session.accessCount.get() > 0) {
-                                // Session is currently being accessed - skip it
-                                continue;
-                            }
+                            // Session is currently being accessed - skip it
+                            continue;
+                        }
                         if (log.isDebugEnabled())
                             log.debug(sm.getString
                                 ("persistentManager.swapMaxIdle",
-                                 session.getIdInternal(), new Integer(timeIdle)));
+                                 session.getIdInternal(),
+                                 Integer.valueOf(timeIdle)));
                         try {
                             swapOut(session);
                         } catch (IOException e) {
-                            ;   // This is logged in writeSession()
+                            // This is logged in writeSession()
                         }
                     }
                 }
@@ -1125,7 +915,7 @@ public abstract class PersistentManagerBase
      */
     protected void processMaxActiveSwaps() {
 
-        if (!isStarted() || getMaxActiveSessions() < 0)
+        if (!getState().isAvailable() || getMaxActiveSessions() < 0)
             return;
 
         Session sessions[] = findSessions();
@@ -1137,7 +927,7 @@ public abstract class PersistentManagerBase
         if(log.isDebugEnabled())
             log.debug(sm.getString
                 ("persistentManager.tooManyActive",
-                 new Integer(sessions.length)));
+                 Integer.valueOf(sessions.length)));
 
         int toswap = sessions.length - getMaxActiveSessions();
         long timeNow = System.currentTimeMillis();
@@ -1146,21 +936,22 @@ public abstract class PersistentManagerBase
             StandardSession session =  (StandardSession) sessions[i];
             synchronized (session) {
                 int timeIdle = // Truncate, do not round up
-                    (int) ((timeNow - session.getLastAccessedTime()) / 1000L);
+                    (int) ((timeNow - session.getThisAccessedTime()) / 1000L);
                 if (timeIdle > minIdleSwap) {
                     if (session.accessCount != null &&
                             session.accessCount.get() > 0) {
-                            // Session is currently being accessed - skip it
-                            continue;
-                        }
+                        // Session is currently being accessed - skip it
+                        continue;
+                    }
                     if(log.isDebugEnabled())
                         log.debug(sm.getString
                             ("persistentManager.swapTooManyActive",
-                             session.getIdInternal(), new Integer(timeIdle)));
+                             session.getIdInternal(),
+                             Integer.valueOf(timeIdle)));
                     try {
                         swapOut(session);
                     } catch (IOException e) {
-                        ;   // This is logged in writeSession()
+                        // This is logged in writeSession()
                     }
                     toswap--;
                 }
@@ -1175,7 +966,7 @@ public abstract class PersistentManagerBase
      */
     protected void processMaxIdleBackups() {
 
-        if (!isStarted() || maxIdleBackup < 0)
+        if (!getState().isAvailable() || maxIdleBackup < 0)
             return;
 
         Session sessions[] = findSessions();
@@ -1189,17 +980,18 @@ public abstract class PersistentManagerBase
                     if (!session.isValid())
                         continue;
                     int timeIdle = // Truncate, do not round up
-                        (int) ((timeNow - session.getLastAccessedTime()) / 1000L);
+                        (int) ((timeNow - session.getThisAccessedTime()) / 1000L);
                     if (timeIdle > maxIdleBackup) {
                         if (log.isDebugEnabled())
                             log.debug(sm.getString
                                 ("persistentManager.backupMaxIdle",
-                                session.getIdInternal(), new Integer(timeIdle)));
+                                session.getIdInternal(),
+                                Integer.valueOf(timeIdle)));
     
                         try {
                             writeSession(session);
                         } catch (IOException e) {
-                            ;   // This is logged in writeSession()
+                            // This is logged in writeSession()
                         }
                     }
                 }
