@@ -20,10 +20,12 @@ package org.apache.catalina.ssi;
 import java.text.ParseException;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.regex.Pattern;
+import java.util.regex.PatternSyntaxException;
 /**
  * Represents a parsed expression.
  * 
- * @version $Revision: 833130 $
+ * @version $Revision: 1136399 $
  * @author Paul Speed
  */
 public class ExpressionParseTree {
@@ -350,6 +352,26 @@ public class ExpressionParseTree {
         protected int compareBranches() {
             String val1 = ((StringNode)left).getValue();
             String val2 = ((StringNode)right).getValue();
+            
+            int val2Len = val2.length();
+            if (val2Len > 1 && val2.charAt(0) == '/' &&
+                    val2.charAt(val2Len - 1) == '/') {
+                // Treat as a regular expression
+                String expr = val2.substring(1, val2Len - 1);
+                try {
+                    Pattern pattern = Pattern.compile(expr);
+                    // Regular expressions will only ever be used with EqualNode
+                    // so return zero for equal and non-zero for not equal
+                    if (pattern.matcher(val1).find()) {
+                        return 0;
+                    } else {
+                        return -1;
+                    }
+                } catch (PatternSyntaxException pse) {
+                    ssiMediator.log("Invalid expression: " + expr, pse);
+                    return 0;
+                }
+            }
             return val1.compareTo(val2);
         }
     }
