@@ -25,6 +25,7 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.nio.ByteBuffer;
+import java.util.concurrent.atomic.AtomicInteger;
 
 /**
  * The XByteBuffer provides a dual functionality.
@@ -42,34 +43,24 @@ import java.nio.ByteBuffer;
  * <li><b>END_DATA</b>  - 7 bytes - <i>TLF2003</i></lI>
  * </ul>
  * @author Filip Hanik
- * @version $Revision: 467173 $, $Date: 2006-10-24 01:12:17 +0200 (Di, 24. Okt 2006) $
+ * @version $Id: XByteBuffer.java 1143022 2011-07-05 12:25:38Z markt $
  */
 public class XByteBuffer
 {
     
-    public static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog( XByteBuffer.class );
     
     /**
      * This is a package header, 7 bytes (FLT2002)
      */
-    public static final byte[] START_DATA = {70,76,84,50,48,48,50};
+    private static final byte[] START_DATA = {70,76,84,50,48,48,50};
     
     /**
      * This is the package footer, 7 bytes (TLF2003)
      */
-    public static final byte[] END_DATA = {84,76,70,50,48,48,51};
+    private static final byte[] END_DATA = {84,76,70,50,48,48,51};
  
-    /**
-     * Default size on the initial byte buffer
-     */
-    private static final int DEF_SIZE = 2048;
- 
-    /**
-     * Default size to extend the buffer with
-     */
-    private static final int DEF_EXT  = 1024;
-    
     /**
      * Variable to hold the data
      */
@@ -92,7 +83,7 @@ public class XByteBuffer
     /**
      * Constructs a new XByteBuffer
      * @param size - the initial size of the byte buffer
-     * @todo use a pool of byte[] for performance
+     * TODO use a pool of byte[] for performance
      */
     public XByteBuffer(int size, boolean discard) {
         buf = new byte[size];
@@ -154,7 +145,6 @@ public class XByteBuffer
      * Appends the data to the buffer. If the data is incorrectly formatted, ie, the data should always start with the
      * header, false will be returned and the data will be discarded.
      * @param b - bytes to be appended
-     * @param off - the offset to extract data from
      * @param len - the number of bytes to append.
      * @return true if the data was appended correctly. Returns false if the package is incorrect, ie missing header or something, or the length of data is 0
      */
@@ -339,10 +329,8 @@ public class XByteBuffer
     
     /**
      * Creates a complete data package
-     * @param indata - the message data to be contained within the package
-     * @param compressed - compression flag for the indata buffer
+     * @param cdata - the message data to be contained within the package
      * @return - a full package (header,size,data,footer)
-     * 
      */
     public static byte[] createDataPackage(ChannelData cdata) {
 //        return createDataPackage(cdata.getDataPackage());
@@ -389,8 +377,7 @@ public class XByteBuffer
         byte[] result = new byte[length];
         return createDataPackage(data,0,data.length,result,0);
     }
-    
-    
+
 
 //    public static void fillDataPackage(byte[] data, int doff, int dlength, XByteBuffer buf) {
 //        int pkglen = getDataPackageLength(dlength);
@@ -406,10 +393,10 @@ public class XByteBuffer
      * @exception java.lang.ArrayIndexOutOfBoundsException
      */
     public static int toInt(byte[] b,int off){
-        return ( ( (int) b[off+3]) & 0xFF) +
-            ( ( ( (int) b[off+2]) & 0xFF) << 8) +
-            ( ( ( (int) b[off+1]) & 0xFF) << 16) +
-            ( ( ( (int) b[off+0]) & 0xFF) << 24);
+        return ( ( b[off+3]) & 0xFF) +
+            ( ( ( b[off+2]) & 0xFF) << 8) +
+            ( ( ( b[off+1]) & 0xFF) << 16) +
+            ( ( ( b[off+0]) & 0xFF) << 24);
     }
 
     /**
@@ -432,11 +419,12 @@ public class XByteBuffer
 
     
     /**
-     * Converts an integer to four bytes
-     * @param n - the integer
-     * @return - four bytes in an array
+     * Converts a boolean to a 1-byte array
+     * @param bool - the integer
+     * @return - 1-byte array
      * @deprecated use toBytes(boolean,byte[],int)
      */
+    @Deprecated
     public static byte[] toBytes(boolean bool) {
         byte[] b = new byte[1] ;
         return toBytes(bool,b,0);
@@ -449,9 +437,10 @@ public class XByteBuffer
     }
     
     /**
-     * 
-     * @param <any> long
-     * @return use
+     * Converts a byte array entry to boolean
+     * @param b byte array
+     * @param offset within byte array
+     * @return true if byte array entry is non-zero, false otherwise
      */
     public static boolean toBoolean(byte[] b, int offset) {
         return b[offset] != 0;
@@ -464,6 +453,7 @@ public class XByteBuffer
      * @return - four bytes in an array
      * @deprecated use toBytes(int,byte[],int)
      */
+    @Deprecated
     public static byte[] toBytes(int n) {
         return toBytes(n,new byte[4],0);
     }
@@ -485,6 +475,7 @@ public class XByteBuffer
      * @return - eight bytes in an array
      * @deprecated use toBytes(long,byte[],int)
      */
+    @Deprecated
     public static byte[] toBytes(long n) {
         return toBytes(n,new byte[8],0);
     }
@@ -562,13 +553,15 @@ public class XByteBuffer
         throws IOException, ClassNotFoundException, ClassCastException {
         return deserialize(data,offset,length,null);     
     }
-    public static int invokecount = 0;
+    
+    private static AtomicInteger invokecount = new AtomicInteger(0);
+    
     public static Serializable deserialize(byte[] data, int offset, int length, ClassLoader[] cls) 
         throws IOException, ClassNotFoundException, ClassCastException {
-        synchronized (XByteBuffer.class) { invokecount++;}
+        invokecount.addAndGet(1);
         Object message = null;
         if ( cls == null ) cls = new ClassLoader[0];
-        if (data != null) {
+        if (data != null && length > 0) {
             InputStream  instream = new ByteArrayInputStream(data,offset,length);
             ObjectInputStream stream = null;
             stream = (cls.length>0)? new ReplicationStream(instream,cls):new ObjectInputStream(instream);
@@ -588,8 +581,7 @@ public class XByteBuffer
     /**
      * Serializes a message into cluster data
      * @param msg ClusterMessage
-     * @param compress boolean
-     * @return 
+     * @return serialized content as byte[] array 
      * @throws IOException
      */
     public static byte[] serialize(Serializable msg) throws IOException {
