@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.util;
 
 import java.io.UnsupportedEncodingException;
@@ -33,9 +31,8 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Tim Tye
- * @version $Id: RequestUtil.java 1157019 2011-08-12 09:11:35Z markt $
+ * @version $Id: RequestUtil.java 1239053 2012-02-01 10:52:00Z markt $
  */
-
 public final class RequestUtil {
 
 
@@ -93,11 +90,18 @@ public final class RequestUtil {
      * try to perform security checks for malicious input.
      *
      * @param path Relative path to be normalized
+     *
+     * @deprecated Deprecated to resolve a circular package dependency and will
+     *             be removed in Tomcat 8.0.x. Use {@link
+     *             org.apache.tomcat.util.http.RequestUtil#normalize(String)} as
+     *             a replacement.
      */
+    @Deprecated
     public static String normalize(String path) {
-        return normalize(path, true);
+        return org.apache.tomcat.util.http.RequestUtil.normalize(path);
     }
 
+
     /**
      * Normalize a relative URI path that may have relative values ("/./",
      * "/../", and so on ) it it.  <strong>WARNING</strong> - This method is
@@ -106,58 +110,16 @@ public final class RequestUtil {
      *
      * @param path Relative path to be normalized
      * @param replaceBackSlash Should '\\' be replaced with '/'
+     *
+     * @deprecated Deprecated to resolve a circular package dependency and will
+     *             be removed in Tomcat 8.0.x. Use {@link
+     *             org.apache.tomcat.util.http.RequestUtil#normalize(String,
+     *             boolean)} as a replacement.
      */
+    @Deprecated
     public static String normalize(String path, boolean replaceBackSlash) {
-
-        if (path == null)
-            return null;
-
-        // Create a place for the normalized path
-        String normalized = path;
-
-        if (replaceBackSlash && normalized.indexOf('\\') >= 0)
-            normalized = normalized.replace('\\', '/');
-
-        if (normalized.equals("/."))
-            return "/";
-
-        // Add a leading "/" if necessary
-        if (!normalized.startsWith("/"))
-            normalized = "/" + normalized;
-
-        // Resolve occurrences of "//" in the normalized path
-        while (true) {
-            int index = normalized.indexOf("//");
-            if (index < 0)
-                break;
-            normalized = normalized.substring(0, index) +
-                normalized.substring(index + 1);
-        }
-
-        // Resolve occurrences of "/./" in the normalized path
-        while (true) {
-            int index = normalized.indexOf("/./");
-            if (index < 0)
-                break;
-            normalized = normalized.substring(0, index) +
-                normalized.substring(index + 2);
-        }
-
-        // Resolve occurrences of "/../" in the normalized path
-        while (true) {
-            int index = normalized.indexOf("/../");
-            if (index < 0)
-                break;
-            if (index == 0)
-                return (null);  // Trying to go outside our context
-            int index2 = normalized.lastIndexOf('/', index - 1);
-            normalized = normalized.substring(0, index2) +
-                normalized.substring(index + 3);
-        }
-
-        // Return the normalized path that we have completed
-        return (normalized);
-
+        return org.apache.tomcat.util.http.RequestUtil.normalize(path,
+                replaceBackSlash);
     }
 
 
