@@ -22,13 +22,15 @@ package org.apache.catalina.util;
 import java.io.InputStream;
 import java.util.Properties;
 
+import org.apache.tomcat.util.ExceptionUtils;
+
 
 /**
  * Simple utility module to make it easy to plug in the server identifier
  * when integrating Tomcat.
  *
  * @author Craig R. McClanahan
- * @version $Id: ServerInfo.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: ServerInfo.java 1002556 2010-09-29 10:07:10Z markt $
  */
 
 public class ServerInfo {
@@ -64,14 +66,14 @@ public class ServerInfo {
             serverBuilt = props.getProperty("server.built");
             serverNumber = props.getProperty("server.number");
         } catch (Throwable t) {
-            ;
+            ExceptionUtils.handleThrowable(t);
         }
         if (serverInfo == null)
-            serverInfo = "Apache Tomcat";
+            serverInfo = "Apache Tomcat 7.0.x-dev";
         if (serverBuilt == null)
             serverBuilt = "unknown";
         if (serverNumber == null)
-            serverNumber = "5.5.0.0";
+            serverNumber = "7.0.x";
         
     }
 
