@@ -258,17 +258,8 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
      * connection.
      */
     public void recycle() {
-        // Recycle filters
-        for (int i = 0; i <= lastActiveFilter; i++) {
-            activeFilters[i].recycle();
-        }
-        // Recycle Request object
-        response.recycle();
-        pos = 0;
-        lastActiveFilter = -1;
-        committed = false;
-        finished = false;
-
+        // Sub-classes may wish to do more than this.
+        nextRequest();
     }
     
     /**
@@ -278,14 +269,12 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
      * to parse the next HTTP request.
      */
     public void nextRequest() {
-
-        // Recycle Request object
-        response.recycle();
         // Recycle filters
         for (int i = 0; i <= lastActiveFilter; i++) {
             activeFilters[i].recycle();
         }
-
+        // Recycle response object
+        response.recycle();
         // Reset pointers
         pos = 0;
         lastActiveFilter = -1;
@@ -449,6 +438,7 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
 
         // Writing the byte chunk to the output buffer
         int length = bc.getLength();
+        checkLengthBeforeWrite(length);
         System.arraycopy(bc.getBytes(), bc.getStart(), buf, pos, length);
         pos = pos + length;
 
@@ -466,6 +456,7 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
 
         int start = cc.getStart();
         int end = cc.getEnd();
+        checkLengthBeforeWrite(end-start);
         char[] cbuf = cc.getBuffer();
         for (int i = start; i < end; i++) {
             char c = cbuf[i];
@@ -490,6 +481,7 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
      * @param b data to be written
      */
     public void write(byte[] b) {
+        checkLengthBeforeWrite(b.length);
 
         // Writing the byte chunk to the output buffer
         System.arraycopy(b, 0, buf, pos, b.length);
@@ -512,6 +504,7 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
 
         // From the Tomcat 3.3 HTTP/1.0 connector
         int len = s.length();
+        checkLengthBeforeWrite(len);
         for (int i = 0; i < len; i++) {
             char c = s.charAt (i);
             // Note:  This is clearly incorrect for many strings,
@@ -541,4 +534,16 @@ public abstract class AbstractOutputBuffer<S> implements OutputBuffer{
     }
 
 
+    /**
+     * Checks to see if there is enough space in the buffer to write the
+     * requested number of bytes.
+     */
+    private void checkLengthBeforeWrite(int length)
+            throws IllegalStateException {
+        if (pos + length > buf.length) {
+            throw new IllegalStateException(
+                    sm.getString("iob.responseheadertoolarge.error"));
+        }
+    }
+
 }
