@@ -90,8 +90,8 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
         
         class CompilationUnit implements ICompilationUnit {
 
-            String className;
-            String sourceFile;
+            private final String className;
+            private final String sourceFile;
 
             CompilationUnit(String sourceFile, String className) {
                 this.className = className;
@@ -107,12 +107,13 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             public char[] getContents() {
                 char[] result = null;
                 FileInputStream is = null;
+                InputStreamReader isr = null;
                 Reader reader = null;
                 try {
                     is = new FileInputStream(sourceFile);
-                     
-                    reader = new BufferedReader(new InputStreamReader(is,
-                            ctxt.getOptions().getJavaEncoding()));
+                    isr = new InputStreamReader(is,
+                            ctxt.getOptions().getJavaEncoding());
+                    reader = new BufferedReader(isr);
                     char[] chars = new char[8192];
                     StringBuilder buf = new StringBuilder();
                     int count;
@@ -130,6 +131,11 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                             reader.close();
                         } catch (IOException ioe) {/*Ignore*/}
                     }
+                    if (isr != null) {
+                        try {
+                            isr.close();
+                        } catch (IOException ioe) {/*Ignore*/}
+                    }
                     if (is != null) {
                         try {
                             is.close();
@@ -159,6 +165,12 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 }
                 return result;
             }
+
+            @SuppressWarnings("unused") // New method added to interface in
+                                        // later JDT versions
+            public boolean ignoreOptionalProblems() {
+                return false;
+            }
         }
 
         final INameEnvironment env = new INameEnvironment() {
