@@ -20,7 +20,6 @@ import java.io.CharArrayWriter;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
-
 import java.util.Iterator;
 import java.util.List;
 import java.util.jar.JarFile;
@@ -55,7 +54,6 @@ class JspDocumentParser
     extends DefaultHandler
     implements LexicalHandler, TagConstants {
 
-    private static final String JSP_VERSION = "version";
     private static final String LEXICAL_HANDLER_PROPERTY =
         "http://xml.org/sax/properties/lexical-handler";
     private static final String JSP_URI = "http://java.sun.com/JSP/Page";
@@ -64,7 +62,7 @@ class JspDocumentParser
     private JspCompilationContext ctxt;
     private PageInfo pageInfo;
     private String path;
-    private StringBuffer charBuffer;
+    private StringBuilder charBuffer;
 
     // Node representing the XML element currently being parsed
     private Node current;
@@ -102,7 +100,7 @@ class JspDocumentParser
 
     // Nesting level of Tag dependent bodies
     private int tagDependentNesting = 0;
-    // Flag set to delay incrmenting tagDependentNesting until jsp:body
+    // Flag set to delay incrementing tagDependentNesting until jsp:body
     // is first encountered
     private boolean tagDependentPending = false;
 
@@ -223,11 +221,11 @@ class JspDocumentParser
      * This is used to implement the include-prelude and include-coda
      * subelements of the jsp-config element in web.xml
      */
-    private void addInclude(Node parent, List files) throws SAXException {
+    private void addInclude(Node parent, List<String> files) throws SAXException {
         if (files != null) {
-            Iterator iter = files.iterator();
+            Iterator<String> iter = files.iterator();
             while (iter.hasNext()) {
-                String file = (String)iter.next();
+                String file = iter.next();
                 AttributesImpl attrs = new AttributesImpl();
                 attrs.addAttribute("", "file", "file", "CDATA", file);
 
@@ -253,6 +251,7 @@ class JspDocumentParser
      * corresponding TagLibraryInfo object is added to the set of custom
      * tag libraries.
      */
+    @Override
     public void startElement(
         String uri,
         String localName,
@@ -352,8 +351,7 @@ class JspDocumentParser
                     nonTaglibAttrs,
                     nonTaglibXmlnsAttrs,
                     taglibAttrs,
-                    startMark,
-                    current);
+                    startMark);
             return;
         }
 
@@ -366,8 +364,7 @@ class JspDocumentParser
                     nonTaglibAttrs,
                     nonTaglibXmlnsAttrs,
                     taglibAttrs,
-                    startMark,
-                    current);
+                    startMark);
             return;
         }
 
@@ -394,8 +391,7 @@ class JspDocumentParser
                     nonTaglibAttrs,
                     nonTaglibXmlnsAttrs,
                     taglibAttrs,
-                    startMark,
-                    current);
+                    startMark);
         } else {
             node =
                 parseCustomAction(
@@ -450,10 +446,11 @@ class JspDocumentParser
      *
      * @throws SAXException
      */
+    @Override
     public void characters(char[] buf, int offset, int len) {
 
         if (charBuffer == null) {
-            charBuffer = new StringBuffer();
+            charBuffer = new StringBuilder();
         }
         charBuffer.append(buf, offset, len);
     }
@@ -606,6 +603,7 @@ class JspDocumentParser
     /*
      * Receives notification of the end of an element.
      */
+    @Override
     public void endElement(String uri, String localName, String qName)
         throws SAXException {
 
@@ -661,21 +659,21 @@ class JspDocumentParser
         }
 
         if (current instanceof Node.CustomTag) {
-        	String bodyType = getBodyType((Node.CustomTag) current);
-        	if (TagInfo.BODY_CONTENT_EMPTY.equalsIgnoreCase(bodyType)) {
-        		// Children - if any - must be JSP attributes
-        		Node.Nodes children = current.getBody();
-        		if (children != null && children.size() > 0) {
-        			for (int i = 0; i < children.size(); i++) {
-        				Node child = children.getNode(i);
-        				if (!(child instanceof Node.NamedAttribute)) {
-        					throw new SAXParseException(Localizer.getMessage(
-        							"jasper.error.emptybodycontent.nonempty",
-        							current.qName), locator); 
-        				}
-        			}
-        		}
-        	}
+            String bodyType = getBodyType((Node.CustomTag) current);
+            if (TagInfo.BODY_CONTENT_EMPTY.equalsIgnoreCase(bodyType)) {
+                // Children - if any - must be JSP attributes
+                Node.Nodes children = current.getBody();
+                if (children != null && children.size() > 0) {
+                    for (int i = 0; i < children.size(); i++) {
+                        Node child = children.getNode(i);
+                        if (!(child instanceof Node.NamedAttribute)) {
+                            throw new SAXParseException(Localizer.getMessage(
+                                    "jasper.error.emptybodycontent.nonempty",
+                                    current.qName), locator); 
+                        }
+                    }
+                }
+            }
         }
         if (current.getParent() != null) {
             current = current.getParent();
@@ -687,6 +685,7 @@ class JspDocumentParser
      *
      * @param locator the document locator
      */
+    @Override
     public void setDocumentLocator(Locator locator) {
         this.locator = locator;
     }
@@ -694,6 +693,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void comment(char[] buf, int offset, int len) throws SAXException {
 
         processChars();  // Flush char buffer and remove white spaces
@@ -713,6 +713,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void startCDATA() throws SAXException {
 
         processChars();  // Flush char buffer and remove white spaces
@@ -723,6 +724,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void endCDATA() throws SAXException {
         processChars();  // Flush char buffer and remove white spaces
     }
@@ -730,6 +732,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void startEntity(String name) throws SAXException {
         // do nothing
     }
@@ -737,6 +740,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void endEntity(String name) throws SAXException {
         // do nothing
     }
@@ -744,6 +748,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void startDTD(String name, String publicId, String systemId)
         throws SAXException {
         if (!isValidating) {
@@ -757,6 +762,7 @@ class JspDocumentParser
     /*
      * See org.xml.sax.ext.LexicalHandler.
      */
+    @Override
     public void endDTD() throws SAXException {
         inDTD = false;
     }
@@ -764,6 +770,7 @@ class JspDocumentParser
     /*
      * Receives notification of a non-recoverable error.
      */
+    @Override
     public void fatalError(SAXParseException e) throws SAXException {
         throw e;
     }
@@ -771,6 +778,7 @@ class JspDocumentParser
     /*
      * Receives notification of a recoverable error.
      */
+    @Override
     public void error(SAXParseException e) throws SAXException {
         throw e;
     }
@@ -778,6 +786,7 @@ class JspDocumentParser
     /*
      * Receives notification of the start of a Namespace mapping. 
      */
+    @Override
     public void startPrefixMapping(String prefix, String uri)
         throws SAXException {
         TagLibraryInfo taglibInfo;
@@ -808,6 +817,7 @@ class JspDocumentParser
     /*
      * Receives notification of the end of a Namespace mapping. 
      */
+    @Override
     public void endPrefixMapping(String prefix) throws SAXException {
 
         if (directivesOnly) {
@@ -829,8 +839,7 @@ class JspDocumentParser
         Attributes nonTaglibAttrs,
         Attributes nonTaglibXmlnsAttrs,
         Attributes taglibAttrs,
-        Mark start,
-        Node parent)
+        Mark start)
         throws SAXException {
 
         Node node = null;
@@ -1179,7 +1188,7 @@ class JspDocumentParser
             throw new SAXException(
                 Localizer.getMessage("jsp.error.xml.bad_tag", localName, uri));
         }
-        Class tagHandlerClass = null;
+        Class<?> tagHandlerClass = null;
         if (tagInfo != null) {
             String handlerClassName = tagInfo.getTagClassName();
             try {
@@ -1264,10 +1273,10 @@ class JspDocumentParser
                 isPlainUri = true;
             }
 
-            String[] location = ctxt.getTldLocation(uri);
+            TldLocation location = ctxt.getTldLocation(uri);
             if (location != null || !isPlainUri) {
                 if (ctxt.getOptions().isCaching()) {
-                    result = (TagLibraryInfoImpl) ctxt.getOptions().getCache().get(uri);
+                    result = ctxt.getOptions().getCache().get(uri);
                 }
                 if (result == null) {
                     /*
@@ -1439,7 +1448,9 @@ class JspDocumentParser
      * validation is turned off.
      */
     private static class EnableDTDValidationException
-        extends SAXParseException {
+            extends SAXParseException {
+
+        private static final long serialVersionUID = 1L;
 
         EnableDTDValidationException(String message, Locator loc) {
             super(message, loc);
