@@ -24,7 +24,7 @@ import java.util.Hashtable;
  * Handles the access control on the JNDI contexts.
  *
  * @author Remy Maucherat
- * @version $Id: ContextAccessController.java 939511 2010-04-29 23:48:20Z kkolinko $
+ * @version $Id: ContextAccessController.java 1078850 2011-03-07 16:46:45Z markt $
  */
 
 public class ContextAccessController {
@@ -36,13 +36,15 @@ public class ContextAccessController {
     /**
      * Catalina context names on which writing is not allowed.
      */
-    private static Hashtable readOnlyContexts = new Hashtable();
+    private static Hashtable<Object,Object> readOnlyContexts =
+        new Hashtable<Object,Object>();
 
 
     /**
      * Security tokens repository.
      */
-    private static Hashtable securityTokens = new Hashtable();
+    private static Hashtable<Object,Object> securityTokens =
+        new Hashtable<Object,Object>();
 
 
     // --------------------------------------------------------- Public Methods
@@ -85,11 +87,7 @@ public class ContextAccessController {
     public static boolean checkSecurityToken
         (Object name, Object token) {
         Object refToken = securityTokens.get(name);
-        if (refToken == null)
-            return (true);
-        if ((refToken != null) && (refToken.equals(token)))
-            return (true);
-        return (false);
+        return (refToken == null || refToken.equals(token));
     }
 
 
