@@ -5,15 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- */ 
+ */
 
 
 package org.apache.naming;
@@ -32,7 +32,7 @@ import javax.naming.NamingException;
  * Catalina JNDI Context implementation.
  *
  * @author Remy Maucherat
- * @version $Id: SelectorContext.java 1026057 2010-10-21 16:26:50Z markt $
+ * @version $Id: SelectorContext.java 1304471 2012-03-23 16:41:38Z kkolinko $
  */
 
 public class SelectorContext implements Context {
@@ -111,11 +111,11 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Retrieves the named object. If name is empty, returns a new instance 
-     * of this context (which represents the same naming context as this 
-     * context, but its environment may be modified independently and it may 
+     * Retrieves the named object. If name is empty, returns a new instance
+     * of this context (which represents the same naming context as this
+     * context, but its environment may be modified independently and it may
      * be accessed concurrently).
-     * 
+     *
      * @param name the name of the object to look up
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
@@ -123,7 +123,7 @@ public class SelectorContext implements Context {
     @Override
     public Object lookup(Name name)
         throws NamingException {
-        
+
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("selectorContext.methodUsingName", "lookup",
                     name));
@@ -138,7 +138,7 @@ public class SelectorContext implements Context {
 
     /**
      * Retrieves the named object.
-     * 
+     *
      * @param name the name of the object to look up
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
@@ -160,10 +160,10 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Binds a name to an object. All intermediate contexts and the target 
-     * context (that named by all but terminal atomic component of the name) 
+     * Binds a name to an object. All intermediate contexts and the target
+     * context (that named by all but terminal atomic component of the name)
      * must already exist.
-     * 
+     *
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -181,7 +181,7 @@ public class SelectorContext implements Context {
 
     /**
      * Binds a name to an object.
-     * 
+     *
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -198,14 +198,14 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Binds a name to an object, overwriting any existing binding. All 
-     * intermediate contexts and the target context (that named by all but 
+     * Binds a name to an object, overwriting any existing binding. All
+     * intermediate contexts and the target context (that named by all but
      * terminal atomic component of the name) must already exist.
      * <p>
-     * If the object is a DirContext, any existing attributes associated with 
-     * the name are replaced with those of the object. Otherwise, any 
+     * If the object is a DirContext, any existing attributes associated with
+     * the name are replaced with those of the object. Otherwise, any
      * existing attributes associated with the name remain unchanged.
-     * 
+     *
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception javax.naming.InvalidAttributesException if object did not
@@ -221,7 +221,7 @@ public class SelectorContext implements Context {
 
     /**
      * Binds a name to an object, overwriting any existing binding.
-     * 
+     *
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception javax.naming.InvalidAttributesException if object did not
@@ -236,14 +236,14 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Unbinds the named object. Removes the terminal atomic name in name 
-     * from the target context--that named by all but the terminal atomic 
+     * Unbinds the named object. Removes the terminal atomic name in name
+     * from the target context--that named by all but the terminal atomic
      * part of name.
      * <p>
-     * This method is idempotent. It succeeds even if the terminal atomic 
-     * name is not bound in the target context, but throws 
-     * NameNotFoundException if any of the intermediate contexts do not exist. 
-     * 
+     * This method is idempotent. It succeeds even if the terminal atomic
+     * name is not bound in the target context, but throws
+     * NameNotFoundException if any of the intermediate contexts do not exist.
+     *
      * @param name the name to bind; may not be empty
      * @exception javax.naming NameNotFoundException if an intermediate context
      * does not exist
@@ -258,7 +258,7 @@ public class SelectorContext implements Context {
 
     /**
      * Unbinds the named object.
-     * 
+     *
      * @param name the name to bind; may not be empty
      * @exception javax.naming NameNotFoundException if an intermediate context
      * does not exist
@@ -272,11 +272,11 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Binds a new name to the object bound to an old name, and unbinds the 
-     * old name. Both names are relative to this context. Any attributes 
-     * associated with the old name become associated with the new name. 
+     * Binds a new name to the object bound to an old name, and unbinds the
+     * old name. Both names are relative to this context. Any attributes
+     * associated with the old name become associated with the new name.
      * Intermediate contexts of the old name are not changed.
-     * 
+     *
      * @param oldName the name of the existing binding; may not be empty
      * @param newName the name of the new binding; may not be empty
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -291,9 +291,9 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Binds a new name to the object bound to an old name, and unbinds the 
+     * Binds a new name to the object bound to an old name, and unbinds the
      * old name.
-     * 
+     *
      * @param oldName the name of the existing binding; may not be empty
      * @param newName the name of the new binding; may not be empty
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -308,22 +308,22 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Enumerates the names bound in the named context, along with the class 
-     * names of objects bound to them. The contents of any subcontexts are 
+     * Enumerates the names bound in the named context, along with the class
+     * names of objects bound to them. The contents of any subcontexts are
      * not included.
      * <p>
-     * If a binding is added to or removed from this context, its effect on 
+     * If a binding is added to or removed from this context, its effect on
      * an enumeration previously returned is undefined.
-     * 
+     *
      * @param name the name of the context to list
-     * @return an enumeration of the names and class names of the bindings in 
+     * @return an enumeration of the names and class names of the bindings in
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
     @Override
     public NamingEnumeration<NameClassPair> list(Name name)
         throws NamingException {
-        
+
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("selectorContext.methodUsingName", "list",
                     name));
@@ -334,18 +334,18 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Enumerates the names bound in the named context, along with the class 
+     * Enumerates the names bound in the named context, along with the class
      * names of objects bound to them.
-     * 
+     *
      * @param name the name of the context to list
-     * @return an enumeration of the names and class names of the bindings in 
+     * @return an enumeration of the names and class names of the bindings in
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
     @Override
     public NamingEnumeration<NameClassPair> list(String name)
         throws NamingException {
-        
+
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("selectorContext.methodUsingString", "list",
                     name));
@@ -356,15 +356,15 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Enumerates the names bound in the named context, along with the 
-     * objects bound to them. The contents of any subcontexts are not 
+     * Enumerates the names bound in the named context, along with the
+     * objects bound to them. The contents of any subcontexts are not
      * included.
      * <p>
-     * If a binding is added to or removed from this context, its effect on 
+     * If a binding is added to or removed from this context, its effect on
      * an enumeration previously returned is undefined.
-     * 
+     *
      * @param name the name of the context to list
-     * @return an enumeration of the bindings in this context. 
+     * @return an enumeration of the bindings in this context.
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
@@ -382,11 +382,11 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Enumerates the names bound in the named context, along with the 
+     * Enumerates the names bound in the named context, along with the
      * objects bound to them.
-     * 
+     *
      * @param name the name of the context to list
-     * @return an enumeration of the bindings in this context. 
+     * @return an enumeration of the bindings in this context.
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
@@ -404,24 +404,24 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Destroys the named context and removes it from the namespace. Any 
-     * attributes associated with the name are also removed. Intermediate 
+     * Destroys the named context and removes it from the namespace. Any
+     * attributes associated with the name are also removed. Intermediate
      * contexts are not destroyed.
      * <p>
-     * This method is idempotent. It succeeds even if the terminal atomic 
-     * name is not bound in the target context, but throws 
-     * NameNotFoundException if any of the intermediate contexts do not exist. 
-     * 
-     * In a federated naming system, a context from one naming system may be 
-     * bound to a name in another. One can subsequently look up and perform 
-     * operations on the foreign context using a composite name. However, an 
-     * attempt destroy the context using this composite name will fail with 
-     * NotContextException, because the foreign context is not a "subcontext" 
-     * of the context in which it is bound. Instead, use unbind() to remove 
-     * the binding of the foreign context. Destroying the foreign context 
-     * requires that the destroySubcontext() be performed on a context from 
+     * This method is idempotent. It succeeds even if the terminal atomic
+     * name is not bound in the target context, but throws
+     * NameNotFoundException if any of the intermediate contexts do not exist.
+     *
+     * In a federated naming system, a context from one naming system may be
+     * bound to a name in another. One can subsequently look up and perform
+     * operations on the foreign context using a composite name. However, an
+     * attempt destroy the context using this composite name will fail with
+     * NotContextException, because the foreign context is not a "subcontext"
+     * of the context in which it is bound. Instead, use unbind() to remove
+     * the binding of the foreign context. Destroying the foreign context
+     * requires that the destroySubcontext() be performed on a context from
      * the foreign context's "native" naming system.
-     * 
+     *
      * @param name the name of the context to be destroyed; may not be empty
      * @exception javax.naming NameNotFoundException if an intermediate context
      * does not exist
@@ -437,7 +437,7 @@ public class SelectorContext implements Context {
 
     /**
      * Destroys the named context and removes it from the namespace.
-     * 
+     *
      * @param name the name of the context to be destroyed; may not be empty
      * @exception javax.naming NameNotFoundException if an intermediate context
      * does not exist
@@ -452,11 +452,11 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Creates and binds a new context. Creates a new context with the given 
-     * name and binds it in the target context (that named by all but 
-     * terminal atomic component of the name). All intermediate contexts and 
+     * Creates and binds a new context. Creates a new context with the given
+     * name and binds it in the target context (that named by all but
+     * terminal atomic component of the name). All intermediate contexts and
      * the target context must already exist.
-     * 
+     *
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -474,7 +474,7 @@ public class SelectorContext implements Context {
 
     /**
      * Creates and binds a new context.
-     * 
+     *
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
      * @exception javax.naming.NameAlreadyBoundException if name is already
@@ -491,12 +491,12 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Retrieves the named object, following links except for the terminal 
-     * atomic component of the name. If the object bound to name is not a 
+     * Retrieves the named object, following links except for the terminal
+     * atomic component of the name. If the object bound to name is not a
      * link, returns the object itself.
-     * 
+     *
      * @param name the name of the object to look up
-     * @return the object bound to name, not following the terminal link 
+     * @return the object bound to name, not following the terminal link
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
@@ -514,11 +514,11 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Retrieves the named object, following links except for the terminal 
+     * Retrieves the named object, following links except for the terminal
      * atomic component of the name.
-     * 
+     *
      * @param name the name of the object to look up
-     * @return the object bound to name, not following the terminal link 
+     * @return the object bound to name, not following the terminal link
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
@@ -536,16 +536,16 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Retrieves the parser associated with the named context. In a 
-     * federation of namespaces, different naming systems will parse names 
-     * differently. This method allows an application to get a parser for 
-     * parsing names into their atomic components using the naming convention 
-     * of a particular naming system. Within any single naming system, 
-     * NameParser objects returned by this method must be equal (using the 
+     * Retrieves the parser associated with the named context. In a
+     * federation of namespaces, different naming systems will parse names
+     * differently. This method allows an application to get a parser for
+     * parsing names into their atomic components using the naming convention
+     * of a particular naming system. Within any single naming system,
+     * NameParser objects returned by this method must be equal (using the
      * equals() test).
-     * 
+     *
      * @param name the name of the context from which to get the parser
-     * @return a name parser that can parse compound names into their atomic 
+     * @return a name parser that can parse compound names into their atomic
      * components
      * @exception NamingException if a naming exception is encountered
      */
@@ -558,9 +558,9 @@ public class SelectorContext implements Context {
 
     /**
      * Retrieves the parser associated with the named context.
-     * 
+     *
      * @param name the name of the context from which to get the parser
-     * @return a name parser that can parse compound names into their atomic 
+     * @return a name parser that can parse compound names into their atomic
      * components
      * @exception NamingException if a naming exception is encountered
      */
@@ -574,13 +574,13 @@ public class SelectorContext implements Context {
     /**
      * Composes the name of this context with a name relative to this context.
      * <p>
-     * Given a name (name) relative to this context, and the name (prefix) 
-     * of this context relative to one of its ancestors, this method returns 
-     * the composition of the two names using the syntax appropriate for the 
-     * naming system(s) involved. That is, if name names an object relative 
-     * to this context, the result is the name of the same object, but 
+     * Given a name (name) relative to this context, and the name (prefix)
+     * of this context relative to one of its ancestors, this method returns
+     * the composition of the two names using the syntax appropriate for the
+     * naming system(s) involved. That is, if name names an object relative
+     * to this context, the result is the name of the same object, but
      * relative to the ancestor context. None of the names may be null.
-     * 
+     *
      * @param name a name relative to this context
      * @param prefix the name of this context relative to one of its ancestors
      * @return the composition of prefix and name
@@ -596,7 +596,7 @@ public class SelectorContext implements Context {
 
     /**
      * Composes the name of this context with a name relative to this context.
-     * 
+     *
      * @param name a name relative to this context
      * @param prefix the name of this context relative to one of its ancestors
      * @return the composition of prefix and name
@@ -610,10 +610,10 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Adds a new environment property to the environment of this context. If 
+     * Adds a new environment property to the environment of this context. If
      * the property already exists, its value is overwritten.
-     * 
-     * @param propName the name of the environment property to add; may not 
+     *
+     * @param propName the name of the environment property to add; may not
      * be null
      * @param propVal the value of the property to add; may not be null
      * @exception NamingException if a naming exception is encountered
@@ -626,9 +626,9 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Removes an environment property from the environment of this context. 
-     * 
-     * @param propName the name of the environment property to remove; 
+     * Removes an environment property from the environment of this context.
+     *
+     * @param propName the name of the environment property to remove;
      * may not be null
      * @exception NamingException if a naming exception is encountered
      */
@@ -640,12 +640,12 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Retrieves the environment in effect for this context. See class 
-     * description for more details on environment properties. 
-     * The caller should not make any changes to the object returned: their 
-     * effect on the context is undefined. The environment of this context 
+     * Retrieves the environment in effect for this context. See class
+     * description for more details on environment properties.
+     * The caller should not make any changes to the object returned: their
+     * effect on the context is undefined. The environment of this context
      * may be changed using addToEnvironment() and removeFromEnvironment().
-     * 
+     *
      * @return the environment of this context; never null
      * @exception NamingException if a naming exception is encountered
      */
@@ -657,13 +657,13 @@ public class SelectorContext implements Context {
 
 
     /**
-     * Closes this context. This method releases this context's resources 
-     * immediately, instead of waiting for them to be released automatically 
+     * Closes this context. This method releases this context's resources
+     * immediately, instead of waiting for them to be released automatically
      * by the garbage collector.
-     * This method is idempotent: invoking it on a context that has already 
-     * been closed has no effect. Invoking any other method on a closed 
+     * This method is idempotent: invoking it on a context that has already
+     * been closed has no effect. Invoking any other method on a closed
      * context is not allowed, and results in undefined behaviour.
-     * 
+     *
      * @exception NamingException if a naming exception is encountered
      */
     @Override
@@ -676,15 +676,15 @@ public class SelectorContext implements Context {
     /**
      * Retrieves the full name of this context within its own namespace.
      * <p>
-     * Many naming services have a notion of a "full name" for objects in 
-     * their respective namespaces. For example, an LDAP entry has a 
-     * distinguished name, and a DNS record has a fully qualified name. This 
-     * method allows the client application to retrieve this name. The string 
-     * returned by this method is not a JNDI composite name and should not be 
-     * passed directly to context methods. In naming systems for which the 
-     * notion of full name does not make sense, 
+     * Many naming services have a notion of a "full name" for objects in
+     * their respective namespaces. For example, an LDAP entry has a
+     * distinguished name, and a DNS record has a fully qualified name. This
+     * method allows the client application to retrieve this name. The string
+     * returned by this method is not a JNDI composite name and should not be
+     * passed directly to context methods. In naming systems for which the
+     * notion of full name does not make sense,
      * OperationNotSupportedException is thrown.
-     * 
+     *
      * @return this context's name in its own namespace; never null
      * @exception javax.naming.OperationNotSupportedException if the naming
      * system does not have the notion of a full name
@@ -715,7 +715,7 @@ public class SelectorContext implements Context {
             }
             Context initialContext = ContextBindings.getContext(ICName);
             if (initialContext == null) {
-                // Allocating a new context and binding it to the appropriate 
+                // Allocating a new context and binding it to the appropriate
                 // name
                 initialContext = new NamingContext(env, ICName);
                 ContextBindings.bindContext(ICName, initialContext);
@@ -734,14 +734,14 @@ public class SelectorContext implements Context {
 
     /**
      * Strips the URL header.
-     * 
+     *
      * @return the parsed name
-     * @exception NamingException if there is no "java:" header or if no 
+     * @exception NamingException if there is no "java:" header or if no
      * naming context has been bound to this thread
      */
-    protected String parseName(String name) 
+    protected String parseName(String name)
         throws NamingException {
-        
+
         if ((!initialContext) && (name.startsWith(prefix))) {
             return (name.substring(prefixLength));
         } else {
@@ -752,29 +752,35 @@ public class SelectorContext implements Context {
                     (sm.getString("selectorContext.noJavaUrl"));
             }
         }
-        
+
     }
 
 
     /**
      * Strips the URL header.
-     * 
+     *
      * @return the parsed name
-     * @exception NamingException if there is no "java:" header or if no 
+     * @exception NamingException if there is no "java:" header or if no
      * naming context has been bound to this thread
      */
-    protected Name parseName(Name name) 
+    protected Name parseName(Name name)
         throws NamingException {
 
-        if ((!initialContext) && (!name.isEmpty()) 
-            && (name.get(0).equals(prefix))) {
-            return (name.getSuffix(1));
+        if (!initialContext && !name.isEmpty() &&
+                name.get(0).startsWith(prefix)) {
+            if (name.get(0).equals(prefix)) {
+                return name.getSuffix(1);
+            } else {
+                Name result = name.getSuffix(1);
+                result.add(0, name.get(0).substring(prefixLength));
+                return result;
+            }
         } else {
             if (initialContext) {
-                return (name);
+                return name;
             } else {
-                throw new NamingException
-                    (sm.getString("selectorContext.noJavaUrl"));
+                throw new NamingException(
+                        sm.getString("selectorContext.noJavaUrl"));
             }
         }
 
