@@ -72,7 +72,9 @@ public final class HexUtils {
 
     /**
      * Provide a mechanism for ensuring this class is loaded.
+     * @deprecated Unused. Will be removed in Tomcat 8.0.x onwards.
      */
+    @Deprecated
     public static void load() {
         // Nothing to do
     }
