@@ -19,9 +19,11 @@ package org.apache.tomcat.jdbc.pool;
 import java.io.PrintWriter;
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.sql.SQLFeatureNotSupportedException;
 import java.util.Iterator;
 import java.util.Properties;
 import java.util.concurrent.Future;
+import java.util.logging.Logger;
 
 import javax.sql.XAConnection;
 
@@ -47,7 +49,7 @@ public class DataSourceProxy implements PoolConfiguration {
 
     protected volatile ConnectionPool pool = null;
 
-    protected PoolConfiguration poolProperties = null;
+    protected volatile PoolConfiguration poolProperties = null;
 
     public DataSourceProxy() {
         this(new PoolProperties());
@@ -59,12 +61,14 @@ public class DataSourceProxy implements PoolConfiguration {
     }
 
 
+    @SuppressWarnings("unused") // Has to match signature in DataSource
     public boolean isWrapperFor(Class<?> iface) throws SQLException {
         // we are not a wrapper of anything
         return false;
     }
 
 
+    @SuppressWarnings("unused") // Has to match signature in DataSource
     public <T> T unwrap(Class<T> iface) throws SQLException {
         //we can't unwrap anything
         return null;
@@ -173,7 +177,7 @@ public class DataSourceProxy implements PoolConfiguration {
      * {@link javax.sql.DataSource#getConnection()}
      */
     public javax.sql.PooledConnection getPooledConnection(String username,
-                                                String password) throws SQLException {
+            String password) throws SQLException {
         return (javax.sql.PooledConnection) getConnection();
     }
 
@@ -533,10 +537,19 @@ public class DataSourceProxy implements PoolConfiguration {
         this.getPoolProperties().setUseEquals(useEquals);
     }
 
+    /**
+     * no-op
+     * {@link javax.sql.DataSource#getParentLogger}
+     */
+    public Logger getParentLogger() throws SQLFeatureNotSupportedException {
+        throw new SQLFeatureNotSupportedException();
+    }
+
     /**
      * no-op
      * {@link javax.sql.DataSource#getLogWriter}
      */
+    @SuppressWarnings("unused") // Has to match signature in DataSource
     public PrintWriter getLogWriter() throws SQLException {
         return null;
     }
@@ -546,6 +559,7 @@ public class DataSourceProxy implements PoolConfiguration {
      * no-op
      * {@link javax.sql.DataSource#setLogWriter(PrintWriter)}
      */
+    @SuppressWarnings("unused") // Has to match signature in DataSource
     public void setLogWriter(PrintWriter out) throws SQLException {
         // NOOP
     }
@@ -1226,4 +1240,74 @@ public class DataSourceProxy implements PoolConfiguration {
         return getPoolProperties().getRollbackOnReturn();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setUseDisposableConnectionFacade(boolean useDisposableConnectionFacade) {
+        getPoolProperties().setUseDisposableConnectionFacade(useDisposableConnectionFacade);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean getUseDisposableConnectionFacade() {
+        return getPoolProperties().getUseDisposableConnectionFacade();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setLogValidationErrors(boolean logValidationErrors) {
+        getPoolProperties().setLogValidationErrors(logValidationErrors);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean getLogValidationErrors() {
+        return getPoolProperties().getLogValidationErrors();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean getPropagateInterruptState() {
+        return getPoolProperties().getPropagateInterruptState();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setPropagateInterruptState(boolean propagateInterruptState) {
+        getPoolProperties().setPropagateInterruptState(propagateInterruptState);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void purge()  {
+        try {
+            createPool().purge();
+        }catch (SQLException x) {
+            log.error("Unable to purge pool.",x);
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public void purgeOnReturn() {
+        try {
+            createPool().purgeOnReturn();
+        }catch (SQLException x) {
+            log.error("Unable to purge pool.",x);
+        }
+    }
+
 }
