@@ -61,6 +61,10 @@ public abstract class JdbcInterceptor implements InvocationHandler {
      */
     public static final String ISWRAPPERFOR_VAL = "isWrapperFor";
 
+    /**
+     * {@link java.sql.Connection#isValid(boolean)} method name
+     */
+    public static final String ISVALID_VAL = "isValid";
 
     /**
      * Properties for this interceptor.
@@ -70,7 +74,7 @@ public abstract class JdbcInterceptor implements InvocationHandler {
     /**
      * The next interceptor in the chain
      */
-    private JdbcInterceptor next = null;
+    private volatile JdbcInterceptor next = null;
     /**
      * Property that decides how we do string comparison, default is to use
      * {@link String#equals(Object)}. If set to <code>false</code> then the
