@@ -46,6 +46,7 @@ public abstract class SimpleHttpClient {
     public static final String INFO_100 = "HTTP/1.1 100";
     public static final String OK_200 = "HTTP/1.1 200";
     public static final String REDIRECT_302 = "HTTP/1.1 302";
+    public static final String FAIL_400 = "HTTP/1.1 400";
     public static final String FAIL_404 = "HTTP/1.1 404";
     public static final String FAIL_413 = "HTTP/1.1 413";
     public static final String FAIL_50X = "HTTP/1.1 50";
@@ -69,7 +70,7 @@ public abstract class SimpleHttpClient {
     private String responseBody;
     private boolean useContentLength;
 
-    public void setPort(int thePort) {
+    protected void setPort(int thePort) {
         port = thePort;
     }
 
@@ -244,6 +245,10 @@ public abstract class SimpleHttpClient {
         return getResponseLine().startsWith(REDIRECT_302);
     }
 
+    public boolean isResponse400() {
+        return getResponseLine().startsWith(FAIL_400);
+    }
+
     public boolean isResponse404() {
         return getResponseLine().startsWith(FAIL_404);
     }
